# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.pkgutil as pkgutil
import salt.utils.pkg
from salt.exceptions import CommandExecutionError, MinionError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, Mock, patch
from tests.support.unit import TestCase


class PkgutilTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.pkgutil
    """

    def setup_loader_modules(self):
        return {pkgutil: {}}

    # 'refresh_db' function tests: 1

    def test_refresh_db(self):
        """
        Test if it updates the pkgutil repo database (pkgutil -U).
        """
        mock = MagicMock(return_value=0)
        with patch.dict(pkgutil.__salt__, {"cmd.retcode": mock}):
            with patch.object(salt.utils.pkg, "clear_rtag", Mock()):
                self.assertTrue(pkgutil.refresh_db())

    # 'upgrade_available' function tests: 1

    def test_upgrade_available(self):
        """
        Test if there is an upgrade available for a certain package.
        """
        mock = MagicMock(return_value="A\n B\n SAME")
        with patch.dict(pkgutil.__salt__, {"cmd.run_stdout": mock}):
            self.assertEqual(pkgutil.upgrade_available("CSWpython"), "")

        mock = MagicMock(side_effect=["A\n B\n SALT", None])
        with patch.dict(pkgutil.__salt__, {"cmd.run_stdout": mock}):
            self.assertEqual(pkgutil.upgrade_available("CSWpython"), "SALT")

            self.assertEqual(pkgutil.upgrade_available("CSWpython"), "")

    # 'list_upgrades' function tests: 1

    def test_list_upgrades(self):
        """
        Test if it list all available package upgrades on this system.
        """
        mock_run = MagicMock(return_value="A\t B\t SAME")
        mock_ret = MagicMock(return_value=0)
        with patch.dict(
            pkgutil.__salt__, {"cmd.run_stdout": mock_run, "cmd.retcode": mock_ret}
        ):
            with patch.object(salt.utils.pkg, "clear_rtag", Mock()):
                self.assertDictEqual(pkgutil.list_upgrades(), {"A": " B"})

    # 'upgrade' function tests: 1

    def test_upgrade(self):
        """
        Test if it upgrade all of the packages to the latest available version.
        """
        mock_run = MagicMock(return_value="A\t B\t SAME")
        mock_ret = MagicMock(return_value=0)
        mock_pkg = MagicMock(return_value="")
        with patch.dict(
            pkgutil.__salt__,
            {
                "cmd.run_stdout": mock_run,
                "cmd.retcode": mock_ret,
                "pkg_resource.stringify": mock_pkg,
                "pkg_resource.sort_pkglist": mock_pkg,
                "cmd.run_all": mock_ret,
                "cmd.run": mock_run,
            },
        ):
            with patch.dict(pkgutil.__context__, {"pkg.list_pkgs": mock_ret}):
                with patch.object(salt.utils.pkg, "clear_rtag", Mock()):
                    self.assertDictEqual(pkgutil.upgrade(), {})

    # 'list_pkgs' function tests: 1

    def test_list_pkgs(self):
        """
        Test if it list the packages currently installed as a dict.
        """
        mock_run = MagicMock(return_value="A\t B\t SAME")
        mock_ret = MagicMock(return_value=True)
        mock_pkg = MagicMock(return_value="")
        with patch.dict(
            pkgutil.__salt__,
            {
                "cmd.run_stdout": mock_run,
                "cmd.retcode": mock_ret,
                "pkg_resource.stringify": mock_pkg,
                "pkg_resource.sort_pkglist": mock_pkg,
                "cmd.run": mock_run,
            },
        ):
            with patch.dict(pkgutil.__context__, {"pkg.list_pkgs": mock_ret}):
                self.assertDictEqual(
                    pkgutil.list_pkgs(versions_as_list=True, removed=True), {}
                )

            self.assertDictEqual(pkgutil.list_pkgs(), {})

        with patch.dict(pkgutil.__context__, {"pkg.list_pkgs": True}):
            self.assertTrue(pkgutil.list_pkgs(versions_as_list=True))

            mock_pkg = MagicMock(return_value=True)
            with patch.dict(pkgutil.__salt__, {"pkg_resource.stringify": mock_pkg}):
                self.assertTrue(pkgutil.list_pkgs())

    # 'version' function tests: 1

    def test_version(self):
        """
        Test if it returns a version if the package is installed.
        """
        mock_ret = MagicMock(return_value=True)
        with patch.dict(pkgutil.__salt__, {"pkg_resource.version": mock_ret}):
            self.assertTrue(pkgutil.version("CSWpython"))

    # 'latest_version' function tests: 1

    def test_latest_version(self):
        """
        Test if it return the latest version of the named package
        available for upgrade or installation.
        """
        self.assertEqual(pkgutil.latest_version(), "")

        mock_run_all = MagicMock(return_value="A\t B\t SAME")
        mock_run = MagicMock(return_value={"stdout": ""})
        mock_ret = MagicMock(return_value=True)
        mock_pkg = MagicMock(return_value="")
        with patch.dict(
            pkgutil.__salt__,
            {
                "cmd.retcode": mock_ret,
                "pkg_resource.stringify": mock_pkg,
                "pkg_resource.sort_pkglist": mock_pkg,
                "cmd.run_all": mock_run,
                "cmd.run": mock_run_all,
            },
        ):
            with patch.object(salt.utils.pkg, "clear_rtag", Mock()):
                self.assertEqual(pkgutil.latest_version("CSWpython"), "")

                self.assertDictEqual(
                    pkgutil.latest_version("CSWpython", "Python"),
                    {"Python": "", "CSWpython": ""},
                )

    # 'install' function tests: 1

    def test_install(self):
        """
        Test if it install packages using the pkgutil tool.
        """
        mock_pkg = MagicMock(side_effect=MinionError)
        with patch.dict(pkgutil.__salt__, {"pkg_resource.parse_targets": mock_pkg}):
            self.assertRaises(CommandExecutionError, pkgutil.install)

        mock_ret = MagicMock(return_value=True)
        mock_pkg = MagicMock(return_value=[""])
        with patch.dict(pkgutil.__salt__, {"pkg_resource.parse_targets": mock_pkg}):
            with patch.dict(pkgutil.__context__, {"pkg.list_pkgs": mock_ret}):
                self.assertDictEqual(pkgutil.install(), {})

        mock_run = MagicMock(return_value="A\t B\t SAME")
        mock_run_all = MagicMock(return_value={"stdout": ""})
        mock_pkg = MagicMock(return_value=[{"bar": "1.2.3"}])
        with patch.dict(
            pkgutil.__salt__,
            {
                "pkg_resource.parse_targets": mock_pkg,
                "pkg_resource.stringify": mock_pkg,
                "pkg_resource.sort_pkglist": mock_pkg,
                "cmd.run_all": mock_run_all,
                "cmd.run": mock_run,
            },
        ):
            with patch.dict(pkgutil.__context__, {"pkg.list_pkgs": mock_ret}):
                self.assertDictEqual(
                    pkgutil.install(pkgs='["foo", {"bar": "1.2.3"}]'), {}
                )

    # 'remove' function tests: 1

    def test_remove(self):
        """
        Test if it remove a package and all its dependencies
        which are not in use by other packages.
        """
        mock_pkg = MagicMock(side_effect=MinionError)
        with patch.dict(pkgutil.__salt__, {"pkg_resource.parse_targets": mock_pkg}):
            self.assertRaises(CommandExecutionError, pkgutil.remove)

        mock_ret = MagicMock(return_value=True)
        mock_run = MagicMock(return_value="A\t B\t SAME")
        mock_run_all = MagicMock(return_value={"stdout": ""})
        mock_pkg = MagicMock(return_value=[""])
        with patch.dict(
            pkgutil.__salt__,
            {
                "pkg_resource.parse_targets": mock_pkg,
                "pkg_resource.stringify": mock_pkg,
                "pkg_resource.sort_pkglist": mock_pkg,
                "cmd.run_all": mock_run_all,
                "cmd.run": mock_run,
            },
        ):
            with patch.dict(pkgutil.__context__, {"pkg.list_pkgs": mock_ret}):
                self.assertDictEqual(pkgutil.remove(), {})

        mock_pkg = MagicMock(return_value=[{"bar": "1.2.3"}])
        with patch.dict(
            pkgutil.__salt__,
            {
                "pkg_resource.parse_targets": mock_pkg,
                "pkg_resource.stringify": mock_pkg,
                "pkg_resource.sort_pkglist": mock_pkg,
                "cmd.run_all": mock_run_all,
                "cmd.run": mock_run,
            },
        ):
            with patch.dict(pkgutil.__context__, {"pkg.list_pkgs": mock_ret}):
                with patch.object(pkgutil, "list_pkgs", return_value={"bar": "1.2.3"}):
                    self.assertDictEqual(pkgutil.remove(pkgs='["foo", "bar"]'), {})

    # 'purge' function tests: 1

    def test_purge(self):
        """
        Test if it package purges are not supported,
        this function is identical to ``remove()``.
        """
        mock_pkg = MagicMock(side_effect=MinionError)
        with patch.dict(pkgutil.__salt__, {"pkg_resource.parse_targets": mock_pkg}):
            self.assertRaises(CommandExecutionError, pkgutil.purge)



# -*- coding: utf-8 -*-
"""
Module for running arbitrary tests
"""

# Import Python libs
from __future__ import absolute_import


def jinja_error():
    """

    CLI Example:

    .. code-block:: bash

        salt '*' salttest.jinja_error
    """
    raise Exception("hehehe")



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import datetime
import hashlib
import os
import textwrap

import salt.utils.files
from salt.ext import six
from tests.support.case import ModuleCase
from tests.support.helpers import slowTest, with_tempfile
from tests.support.mixins import SaltReturnAssertsMixin
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import skipIf

try:
    import M2Crypto  # pylint: disable=W0611

    HAS_M2CRYPTO = True
except ImportError:
    HAS_M2CRYPTO = False


@skipIf(not HAS_M2CRYPTO, "Skip when no M2Crypto found")
class x509Test(ModuleCase, SaltReturnAssertsMixin):
    @classmethod
    def setUpClass(cls):
        cert_path = os.path.join(RUNTIME_VARS.BASE_FILES, "x509_test.crt")
        with salt.utils.files.fopen(cert_path) as fp:
            cls.x509_cert_text = fp.read()

    def setUp(self):
        with salt.utils.files.fopen(
            os.path.join(RUNTIME_VARS.TMP_PILLAR_TREE, "signing_policies.sls"), "w"
        ) as fp:
            fp.write(
                textwrap.dedent(
                    """\
                x509_signing_policies:
                  ca_policy:
                    - minions: '*'
                    - signing_private_key: {0}/pki/ca.key
                    - signing_cert: {0}/pki/ca.crt
                    - O: Test Company
                    - basicConstraints: "CA:false"
                    - keyUsage: "critical digitalSignature, keyEncipherment"
                    - extendedKeyUsage: "critical serverAuth, clientAuth"
                    - subjectKeyIdentifier: hash
                    - authorityKeyIdentifier: keyid
                    - days_valid: 730
                    - copypath: {0}/pki
                  compound_match:
                    - minions: 'G@x509_test_grain:correct_value'
                    - signing_private_key: {0}/pki/ca.key
                    - signing_cert: {0}/pki/ca.crt
                    - O: Test Company
                    - basicConstraints: "CA:false"
                    - keyUsage: "critical digitalSignature, keyEncipherment"
                    - extendedKeyUsage: "critical serverAuth, clientAuth"
                    - subjectKeyIdentifier: hash
                    - authorityKeyIdentifier: keyid
                    - days_valid: 730
                    - copypath: {0}/pki
                     """.format(
                        RUNTIME_VARS.TMP
                    )
                )
            )
        with salt.utils.files.fopen(
            os.path.join(RUNTIME_VARS.TMP_PILLAR_TREE, "top.sls"), "w"
        ) as fp:
            fp.write(
                textwrap.dedent(
                    """\
                     base:
                       '*':
                         - signing_policies
                     """
                )
            )
        self.run_function("saltutil.refresh_pillar")
        self.run_function(
            "grains.set", ["x509_test_grain", "correct_value"], minion_tgt="sub_minion"
        )
        self.run_function(
            "grains.set", ["x509_test_grain", "not_correct_value"], minion_tgt="minion"
        )

    def tearDown(self):
        os.remove(os.path.join(RUNTIME_VARS.TMP_PILLAR_TREE, "signing_policies.sls"))
        os.remove(os.path.join(RUNTIME_VARS.TMP_PILLAR_TREE, "top.sls"))
        certs_path = os.path.join(RUNTIME_VARS.TMP, "pki")
        if os.path.exists(certs_path):
            salt.utils.files.rm_rf(certs_path)
        self.run_function("saltutil.refresh_pillar")
        self.run_function("grains.delkey", ["x509_test_grain"], minion_tgt="sub_minion")
        self.run_function("grains.delkey", ["x509_test_grain"], minion_tgt="minion")

    def run_function(self, *args, **kwargs):  # pylint: disable=arguments-differ
        ret = super(x509Test, self).run_function(*args, **kwargs)
        return ret

    @staticmethod
    def file_checksum(path):
        hash = hashlib.sha1()
        with salt.utils.files.fopen(path, "rb") as f:
            for block in iter(lambda: f.read(4096), b""):
                hash.update(block)
        return hash.hexdigest()

    @with_tempfile(suffix=".pem", create=False)
    @slowTest
    def test_issue_49027(self, pemfile):
        ret = self.run_state("x509.pem_managed", name=pemfile, text=self.x509_cert_text)
        assert isinstance(ret, dict), ret
        ret = ret[next(iter(ret))]
        assert ret.get("result") is True, ret
        with salt.utils.files.fopen(pemfile) as fp:
            result = fp.readlines()
        self.assertEqual(self.x509_cert_text.splitlines(True), result)

    @with_tempfile(suffix=".crt", create=False)
    @with_tempfile(suffix=".key", create=False)
    @slowTest
    def test_issue_49008(self, keyfile, crtfile):
        ret = self.run_function(
            "state.apply",
            ["issue-49008"],
            pillar={"keyfile": keyfile, "crtfile": crtfile},
        )
        assert isinstance(ret, dict), ret
        for state_result in six.itervalues(ret):
            assert state_result["result"] is True, state_result
        assert os.path.exists(keyfile)
        assert os.path.exists(crtfile)

    @slowTest
    def test_cert_signing(self):
        ret = self.run_function(
            "state.apply", ["x509.cert_signing"], pillar={"tmp_dir": RUNTIME_VARS.TMP}
        )
        key = "x509_|-test_crt_|-{}/pki/test.crt_|-certificate_managed".format(
            RUNTIME_VARS.TMP
        )
        assert key in ret
        assert "changes" in ret[key]
        assert "Certificate" in ret[key]["changes"]
        assert "New" in ret[key]["changes"]["Certificate"]

    def test_cert_issue_not_before_not_after(self):
        ret = self.run_function(
            "state.apply",
            ["test_cert_not_before_not_after"],
            pillar={"tmp_dir": RUNTIME_VARS.TMP},
        )
        key = "x509_|-test_crt_|-{}/pki/test.crt_|-certificate_managed".format(
            RUNTIME_VARS.TMP
        )
        assert key in ret
        assert "changes" in ret[key]
        assert "Certificate" in ret[key]["changes"]
        assert "New" in ret[key]["changes"]["Certificate"]
        assert "Not Before" in ret[key]["changes"]["Certificate"]["New"]
        assert "Not After" in ret[key]["changes"]["Certificate"]["New"]
        not_before = ret[key]["changes"]["Certificate"]["New"]["Not Before"]
        not_after = ret[key]["changes"]["Certificate"]["New"]["Not After"]
        assert not_before == "2019-05-05 00:00:00"
        assert not_after == "2020-05-05 14:30:00"

    def test_cert_issue_not_before(self):
        ret = self.run_function(
            "state.apply",
            ["test_cert_not_before"],
            pillar={"tmp_dir": RUNTIME_VARS.TMP},
        )
        key = "x509_|-test_crt_|-{}/pki/test.crt_|-certificate_managed".format(
            RUNTIME_VARS.TMP
        )
        assert key in ret
        assert "changes" in ret[key]
        assert "Certificate" in ret[key]["changes"]
        assert "New" in ret[key]["changes"]["Certificate"]
        assert "Not Before" in ret[key]["changes"]["Certificate"]["New"]
        assert "Not After" in ret[key]["changes"]["Certificate"]["New"]
        not_before = ret[key]["changes"]["Certificate"]["New"]["Not Before"]
        assert not_before == "2019-05-05 00:00:00"

    def test_cert_issue_not_after(self):
        ret = self.run_function(
            "state.apply", ["test_cert_not_after"], pillar={"tmp_dir": RUNTIME_VARS.TMP}
        )
        key = "x509_|-test_crt_|-{}/pki/test.crt_|-certificate_managed".format(
            RUNTIME_VARS.TMP
        )
        assert key in ret
        assert "changes" in ret[key]
        assert "Certificate" in ret[key]["changes"]
        assert "New" in ret[key]["changes"]["Certificate"]
        assert "Not Before" in ret[key]["changes"]["Certificate"]["New"]
        assert "Not After" in ret[key]["changes"]["Certificate"]["New"]
        not_after = ret[key]["changes"]["Certificate"]["New"]["Not After"]
        assert not_after == "2020-05-05 14:30:00"

    @with_tempfile(suffix=".crt", create=False)
    @with_tempfile(suffix=".key", create=False)
    def test_issue_41858(self, keyfile, crtfile):
        ret_key = "x509_|-test_crt_|-{0}_|-certificate_managed".format(crtfile)
        signing_policy = "no_such_policy"
        ret = self.run_function(
            "state.apply",
            ["issue-41858.gen_cert"],
            pillar={
                "keyfile": keyfile,
                "crtfile": crtfile,
                "tmp_dir": RUNTIME_VARS.TMP,
            },
        )
        self.assertTrue(ret[ret_key]["result"])
        cert_sum = self.file_checksum(crtfile)

        ret = self.run_function(
            "state.apply",
            ["issue-41858.check"],
            pillar={
                "keyfile": keyfile,
                "crtfile": crtfile,
                "signing_policy": signing_policy,
            },
        )
        self.assertFalse(ret[ret_key]["result"])
        # self.assertSaltCommentRegexpMatches(ret[ret_key], "Signing policy {0} does not exist".format(signing_policy))
        self.assertEqual(self.file_checksum(crtfile), cert_sum)

    @with_tempfile(suffix=".crt", create=False)
    @with_tempfile(suffix=".key", create=False)
    def test_compound_match_minion_have_correct_grain_value(self, keyfile, crtfile):
        ret_key = "x509_|-test_crt_|-{0}_|-certificate_managed".format(crtfile)
        signing_policy = "compound_match"
        ret = self.run_function(
            "state.apply",
            ["x509_compound_match.gen_ca"],
            pillar={"tmp_dir": RUNTIME_VARS.TMP},
        )

        # sub_minion have grain set and CA is on other minion
        # CA minion have same grain with incorrect value
        ret = self.run_function(
            "state.apply",
            ["x509_compound_match.check"],
            minion_tgt="sub_minion",
            pillar={
                "keyfile": keyfile,
                "crtfile": crtfile,
                "signing_policy": signing_policy,
            },
        )
        self.assertTrue(ret[ret_key]["result"])

    @with_tempfile(suffix=".crt", create=False)
    @with_tempfile(suffix=".key", create=False)
    def test_compound_match_ca_have_correct_grain_value(self, keyfile, crtfile):
        self.run_function(
            "grains.set", ["x509_test_grain", "correct_value"], minion_tgt="minion"
        )
        self.run_function(
            "grains.set",
            ["x509_test_grain", "not_correct_value"],
            minion_tgt="sub_minion",
        )

        ret_key = "x509_|-test_crt_|-{0}_|-certificate_managed".format(crtfile)
        signing_policy = "compound_match"
        self.run_function(
            "state.apply",
            ["x509_compound_match.gen_ca"],
            pillar={"tmp_dir": RUNTIME_VARS.TMP},
        )

        ret = self.run_function(
            "state.apply",
            ["x509_compound_match.check"],
            minion_tgt="sub_minion",
            pillar={
                "keyfile": keyfile,
                "crtfile": crtfile,
                "signing_policy": signing_policy,
            },
        )
        self.assertFalse(ret[ret_key]["result"])

    @with_tempfile(suffix=".crt", create=False)
    @with_tempfile(suffix=".key", create=False)
    def test_self_signed_cert(self, keyfile, crtfile):
        """
        Self-signed certificate, no CA.
        Run the state twice to confirm the cert is only created once
        and its contents don't change.
        """
        first_run = self.run_function(
            "state.apply",
            ["x509.self_signed"],
            pillar={"keyfile": keyfile, "crtfile": crtfile},
        )
        key = "x509_|-self_signed_cert_|-{}_|-certificate_managed".format(crtfile)
        self.assertIn("New", first_run[key]["changes"]["Certificate"])
        self.assertEqual(
            "Certificate is valid and up to date",
            first_run[key]["changes"]["Status"]["New"],
        )
        self.assertTrue(os.path.exists(crtfile), "Certificate was not created.")

        with salt.utils.files.fopen(crtfile, "r") as first_cert:
            cert_contents = first_cert.read()

        second_run = self.run_function(
            "state.apply",
            ["x509.self_signed"],
            pillar={"keyfile": keyfile, "crtfile": crtfile},
        )
        self.assertEqual({}, second_run[key]["changes"])
        with salt.utils.files.fopen(crtfile, "r") as second_cert:
            self.assertEqual(
                cert_contents,
                second_cert.read(),
                "Certificate contents should not have changed.",
            )

    @with_tempfile(suffix=".crt", create=False)
    @with_tempfile(suffix=".key", create=False)
    def test_old_self_signed_cert_is_recreated(self, keyfile, crtfile):
        """
        Self-signed certificate, no CA.
        First create a cert that expires in 30 days, then recreate
        the cert because the second state run requires days_remaining
        to be at least 90.
        """
        first_run = self.run_function(
            "state.apply",
            ["x509.self_signed_expiry"],
            pillar={
                "keyfile": keyfile,
                "crtfile": crtfile,
                "days_valid": 30,
                "days_remaining": 10,
            },
        )
        key = "x509_|-self_signed_cert_|-{0}_|-certificate_managed".format(crtfile)
        self.assertEqual(
            "Certificate is valid and up to date",
            first_run[key]["changes"]["Status"]["New"],
        )
        expiry = datetime.datetime.strptime(
            first_run[key]["changes"]["Certificate"]["New"]["Not After"],
            "%Y-%m-%d %H:%M:%S",
        )
        self.assertEqual(29, (expiry - datetime.datetime.now()).days)
        self.assertTrue(os.path.exists(crtfile), "Certificate was not created.")

        with salt.utils.files.fopen(crtfile, "r") as first_cert:
            cert_contents = first_cert.read()

        second_run = self.run_function(
            "state.apply",
            ["x509.self_signed_expiry"],
            pillar={
                "keyfile": keyfile,
                "crtfile": crtfile,
                "days_valid": 180,
                "days_remaining": 90,
            },
        )
        self.assertEqual(
            "Certificate needs renewal: 29 days remaining but it needs to be at least 90",
            second_run[key]["changes"]["Status"]["Old"],
        )
        expiry = datetime.datetime.strptime(
            second_run[key]["changes"]["Certificate"]["New"]["Not After"],
            "%Y-%m-%d %H:%M:%S",
        )
        self.assertEqual(179, (expiry - datetime.datetime.now()).days)
        with salt.utils.files.fopen(crtfile, "r") as second_cert:
            self.assertNotEqual(
                cert_contents,
                second_cert.read(),
                "Certificate contents should have changed.",
            )

    @with_tempfile(suffix=".crt", create=False)
    @with_tempfile(suffix=".key", create=False)
    def test_mismatched_self_signed_cert_is_recreated(self, keyfile, crtfile):
        """
        Self-signed certificate, no CA.
        First create a cert, then run the state again with a different
        subjectAltName. The cert should be recreated.
        Finally, run once more with the same subjectAltName as the
        second run. Nothing should change.
        """
        first_run = self.run_function(
            "state.apply",
            ["x509.self_signed_different_properties"],
            pillar={
                "keyfile": keyfile,
                "crtfile": crtfile,
                "subjectAltName": "DNS:alt.service.local",
            },
        )
        key = "x509_|-self_signed_cert_|-{0}_|-certificate_managed".format(crtfile)
        self.assertEqual(
            "Certificate is valid and up to date",
            first_run[key]["changes"]["Status"]["New"],
        )
        sans = first_run[key]["changes"]["Certificate"]["New"]["X509v3 Extensions"][
            "subjectAltName"
        ]
        self.assertEqual("DNS:alt.service.local", sans)
        self.assertTrue(os.path.exists(crtfile), "Certificate was not created.")

        with salt.utils.files.fopen(crtfile, "r") as first_cert:
            first_cert_contents = first_cert.read()

        second_run_pillar = {
            "keyfile": keyfile,
            "crtfile": crtfile,
            "subjectAltName": "DNS:alt1.service.local, DNS:alt2.service.local",
        }
        second_run = self.run_function(
            "state.apply",
            ["x509.self_signed_different_properties"],
            pillar=second_run_pillar,
        )
        self.assertEqual(
            "Certificate properties are different: X509v3 Extensions",
            second_run[key]["changes"]["Status"]["Old"],
        )
        sans = second_run[key]["changes"]["Certificate"]["New"]["X509v3 Extensions"][
            "subjectAltName"
        ]
        self.assertEqual("DNS:alt1.service.local, DNS:alt2.service.local", sans)
        with salt.utils.files.fopen(crtfile, "r") as second_cert:
            second_cert_contents = second_cert.read()
            self.assertNotEqual(
                first_cert_contents,
                second_cert_contents,
                "Certificate contents should have changed.",
            )

        third_run = self.run_function(
            "state.apply",
            ["x509.self_signed_different_properties"],
            pillar=second_run_pillar,
        )
        self.assertEqual({}, third_run[key]["changes"])
        with salt.utils.files.fopen(crtfile, "r") as third_cert:
            self.assertEqual(
                second_cert_contents,
                third_cert.read(),
                "Certificate contents should not have changed.",
            )

    @with_tempfile(suffix=".crt", create=False)
    @with_tempfile(suffix=".key", create=False)
    def test_certificate_managed_with_managed_private_key_does_not_error(
        self, keyfile, crtfile
    ):
        """
        Test using the deprecated managed_private_key arg in certificate_managed does not throw an error.

        TODO: Remove this test in Aluminium when the arg is removed.
        """
        self.run_state("x509.private_key_managed", name=keyfile, bits=4096)
        ret = self.run_state(
            "x509.certificate_managed",
            name=crtfile,
            CN="localhost",
            signing_private_key=keyfile,
            managed_private_key={"name": keyfile, "bits": 4096},
        )
        key = "x509_|-{0}_|-{0}_|-certificate_managed".format(crtfile)
        self.assertEqual(True, ret[key]["result"])



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from tests.support.case import SSHCase
from tests.support.helpers import slowTest


class SSHJinjaFiltersTest(SSHCase):
    """
    testing Jinja filters are available via state system & salt-ssh
    """

    @slowTest
    def test_dateutils_strftime(self):
        """
        test jinja filter datautils.strftime
        """
        arg = self._arg_str("state.sls", ["jinja_filters.dateutils_strftime"])
        ret = self.run_ssh(arg)
        import salt.utils.json

        ret = salt.utils.json.loads(ret)["localhost"]
        self.assertIn("module_|-test_|-test.echo_|-run", ret)
        self.assertIn("ret", ret["module_|-test_|-test.echo_|-run"]["changes"])



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.influxdb08_user as influxdb08_user

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class InfluxdbUserTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.influxdb08_user
    """

    def setup_loader_modules(self):
        return {influxdb08_user: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure that the cluster admin or database user is present.
        """
        name = "salt"
        passwd = "salt"

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        mock = MagicMock(side_effect=[False, False, False, True])
        mock_t = MagicMock(side_effect=[True, False])
        mock_f = MagicMock(return_value=False)
        with patch.dict(
            influxdb08_user.__salt__,
            {
                "influxdb08.db_exists": mock_f,
                "influxdb08.user_exists": mock,
                "influxdb08.user_create": mock_t,
            },
        ):
            comt = "Database mydb does not exist"
            ret.update({"comment": comt})
            self.assertDictEqual(
                influxdb08_user.present(name, passwd, database="mydb"), ret
            )

            with patch.dict(influxdb08_user.__opts__, {"test": True}):
                comt = "User {0} is not present and needs to be created".format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(influxdb08_user.present(name, passwd), ret)

            with patch.dict(influxdb08_user.__opts__, {"test": False}):
                comt = "User {0} has been created".format(name)
                ret.update(
                    {"comment": comt, "result": True, "changes": {"salt": "Present"}}
                )
                self.assertDictEqual(influxdb08_user.present(name, passwd), ret)

                comt = "Failed to create user {0}".format(name)
                ret.update({"comment": comt, "result": False, "changes": {}})
                self.assertDictEqual(influxdb08_user.present(name, passwd), ret)

            comt = "User {0} is already present".format(name)
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(influxdb08_user.present(name, passwd), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure that the named cluster admin or database user is absent.
        """
        name = "salt"

        ret = {"name": name, "result": None, "comment": "", "changes": {}}

        mock = MagicMock(side_effect=[True, True, True, False])
        mock_t = MagicMock(side_effect=[True, False])
        with patch.dict(
            influxdb08_user.__salt__,
            {"influxdb08.user_exists": mock, "influxdb08.user_remove": mock_t},
        ):
            with patch.dict(influxdb08_user.__opts__, {"test": True}):
                comt = "User {0} is present and needs to be removed".format(name)
                ret.update({"comment": comt})
                self.assertDictEqual(influxdb08_user.absent(name), ret)

            with patch.dict(influxdb08_user.__opts__, {"test": False}):
                comt = "User {0} has been removed".format(name)
                ret.update(
                    {"comment": comt, "result": True, "changes": {"salt": "Absent"}}
                )
                self.assertDictEqual(influxdb08_user.absent(name), ret)

                comt = "Failed to remove user {0}".format(name)
                ret.update({"comment": comt, "result": False, "changes": {}})
                self.assertDictEqual(influxdb08_user.absent(name), ret)

            comt = "User {0} is not present, so it cannot be removed".format(name)
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(influxdb08_user.absent(name), ret)



# -*- coding: utf-8 -*-
"""
Tests for salt.modules.zpool

:codeauthor:    Nitin Madhok <nmadhok@clemson.edu>, Jorge Schrauwen <sjorge@blackdot.be>
:maintainer:    Jorge Schrauwen <sjorge@blackdot.be>
:maturity:      new
:depends:       salt.utils.zfs
:platform:      illumos,freebsd,linux
"""

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Utils
import salt.loader
import salt.modules.zpool as zpool
import salt.utils.decorators
import salt.utils.decorators.path

# Import Salt Execution module to test
import salt.utils.zfs
from salt.utils.odict import OrderedDict
from tests.support.helpers import slowTest
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf

# Import Salt Testing libs
from tests.support.zfs import ZFSMockData


# Skip this test case if we don't have access to mock!
class ZpoolTestCase(TestCase, LoaderModuleMockMixin):
    """
    This class contains a set of functions that test salt.modules.zpool module
    """

    def setup_loader_modules(self):
        self.opts = opts = salt.config.DEFAULT_MINION_OPTS.copy()
        self.utils_patch = ZFSMockData().get_patched_utils()
        for key in ("opts", "utils_patch"):
            self.addCleanup(delattr, self, key)
        utils = salt.loader.utils(
            opts, whitelist=["zfs", "args", "systemd", "path", "platform"]
        )
        zpool_obj = {zpool: {"__opts__": opts, "__utils__": utils}}

        return zpool_obj

    @slowTest
    def test_exists_success(self):
        """
        Tests successful return of exists function
        """
        ret = {}
        ret["stdout"] = (
            "NAME      SIZE  ALLOC   FREE    CAP  DEDUP  HEALTH  ALTROOT\n"
            "myzpool   149G   128K   149G     0%  1.00x  ONLINE  -"
        )
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            self.assertTrue(zpool.exists("myzpool"))

    @slowTest
    def test_exists_failure(self):
        """
        Tests failure return of exists function
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'myzpool': no such pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            self.assertFalse(zpool.exists("myzpool"))

    def test_healthy(self):
        """
        Tests successful return of healthy function
        """
        ret = {}
        ret["stdout"] = "all pools are healthy"
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            self.assertTrue(zpool.healthy())

    def test_status(self):
        """
        Tests successful return of status function
        """
        ret = {}
        ret["stdout"] = "\n".join(
            [
                "  pool: mypool",
                " state: ONLINE",
                "  scan: scrub repaired 0 in 0h6m with 0 errors on Mon Dec 21 02:06:17 2015",
                "config:",
                "",
                "\tNAME        STATE     READ WRITE CKSUM",
                "\tmypool      ONLINE       0     0     0",
                "\t  mirror-0  ONLINE       0     0     0",
                "\t    c2t0d0  ONLINE       0     0     0",
                "\t    c2t1d0  ONLINE       0     0     0",
                "",
                "errors: No known data errors",
            ]
        )
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.status()
            self.assertEqual("ONLINE", ret["mypool"]["state"])

    @slowTest
    def test_iostat(self):
        """
        Tests successful return of iostat function
        """
        ret = {}
        ret["stdout"] = "\n".join(
            [
                "               capacity     operations    bandwidth",
                "pool        alloc   free   read  write   read  write",
                "----------  -----  -----  -----  -----  -----  -----",
                "mypool      46.7G  64.3G      4     19   113K   331K",
                "  mirror    46.7G  64.3G      4     19   113K   331K",
                "    c2t0d0      -      -      1     10   114K   334K",
                "    c2t1d0      -      -      1     10   114K   334K",
                "----------  -----  -----  -----  -----  -----  -----",
            ]
        )
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.iostat("mypool", parsable=False)
            self.assertEqual("46.7G", ret["mypool"]["capacity-alloc"])

    def test_iostat_parsable(self):
        """
        Tests successful return of iostat function

        .. note:
            The command output is the same as the non parsable!
            There is no -p flag for zpool iostat, but our type
            conversions can handle this!
        """
        ret = {}
        ret["stdout"] = "\n".join(
            [
                "               capacity     operations    bandwidth",
                "pool        alloc   free   read  write   read  write",
                "----------  -----  -----  -----  -----  -----  -----",
                "mypool      46.7G  64.3G      4     19   113K   331K",
                "  mirror    46.7G  64.3G      4     19   113K   331K",
                "    c2t0d0      -      -      1     10   114K   334K",
                "    c2t1d0      -      -      1     10   114K   334K",
                "----------  -----  -----  -----  -----  -----  -----",
            ]
        )
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.iostat("mypool", parsable=True)
            self.assertEqual(50143743180, ret["mypool"]["capacity-alloc"])

    def test_list(self):
        """
        Tests successful return of list function
        """
        ret = {}
        ret["stdout"] = "mypool\t1.81T\t661G\t1.17T\t35%\t11%\tONLINE"
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.list_(parsable=False)
            res = OrderedDict(
                [
                    (
                        "mypool",
                        OrderedDict(
                            [
                                ("size", "1.81T"),
                                ("alloc", "661G"),
                                ("free", "1.17T"),
                                ("cap", "35%"),
                                ("frag", "11%"),
                                ("health", "ONLINE"),
                            ]
                        ),
                    )
                ]
            )
            self.assertEqual(ret, res)

    @slowTest
    def test_list_parsable(self):
        """
        Tests successful return of list function with parsable output
        """
        ret = {}
        ret["stdout"] = "mypool\t1.81T\t661G\t1.17T\t35%\t11%\tONLINE"
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.list_(parsable=True)
            res = OrderedDict(
                [
                    (
                        "mypool",
                        OrderedDict(
                            [
                                ("size", 1990116046274),
                                ("alloc", 709743345664),
                                ("free", 1286428604497),
                                ("cap", "35%"),
                                ("frag", "11%"),
                                ("health", "ONLINE"),
                            ]
                        ),
                    )
                ]
            )
            self.assertEqual(ret, res)

    def test_get(self):
        """
        Tests successful return of get function
        """
        ret = {}
        ret["stdout"] = "mypool\tsize\t1.81T\t-\n"
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.get("mypool", "size", parsable=False)
            res = OrderedDict(OrderedDict([("size", "1.81T")]))
            self.assertEqual(ret, res)

    @slowTest
    def test_get_parsable(self):
        """
        Tests successful return of get function with parsable output
        """
        ret = {}
        ret["stdout"] = "mypool\tsize\t1.81T\t-\n"
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.get("mypool", "size", parsable=True)
            res = OrderedDict(OrderedDict([("size", 1990116046274)]))
            self.assertEqual(ret, res)

    @slowTest
    def test_get_whitespace(self):
        """
        Tests successful return of get function with a string with whitespaces
        """
        ret = {}
        ret["stdout"] = "mypool\tcomment\tmy testing pool\t-\n"
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.get("mypool", "comment")
            res = OrderedDict(OrderedDict([("comment", "my testing pool")]))
            self.assertEqual(ret, res)

    @slowTest
    def test_scrub_start(self):
        """
        Tests start of scrub
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        mock_exists = MagicMock(return_value=True)

        with patch.dict(zpool.__salt__, {"zpool.exists": mock_exists}), patch.dict(
            zpool.__salt__, {"cmd.run_all": mock_cmd}
        ), patch.dict(zpool.__utils__, self.utils_patch):
            ret = zpool.scrub("mypool")
            res = OrderedDict(OrderedDict([("scrubbing", True)]))
            self.assertEqual(ret, res)

    @slowTest
    def test_scrub_pause(self):
        """
        Tests pause of scrub
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        mock_exists = MagicMock(return_value=True)

        with patch.dict(zpool.__salt__, {"zpool.exists": mock_exists}), patch.dict(
            zpool.__salt__, {"cmd.run_all": mock_cmd}
        ), patch.dict(zpool.__utils__, self.utils_patch):
            ret = zpool.scrub("mypool", pause=True)
            res = OrderedDict(OrderedDict([("scrubbing", False)]))
            self.assertEqual(ret, res)

    @slowTest
    def test_scrub_stop(self):
        """
        Tests pauze of scrub
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)
        mock_exists = MagicMock(return_value=True)

        with patch.dict(zpool.__salt__, {"zpool.exists": mock_exists}), patch.dict(
            zpool.__salt__, {"cmd.run_all": mock_cmd}
        ), patch.dict(zpool.__utils__, self.utils_patch):
            ret = zpool.scrub("mypool", stop=True)
            res = OrderedDict(OrderedDict([("scrubbing", False)]))
            self.assertEqual(ret, res)

    def test_split_success(self):
        """
        Tests split on success
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.split("datapool", "backuppool")
            res = OrderedDict([("split", True)])
            self.assertEqual(ret, res)

    @slowTest
    def test_split_exist_new(self):
        """
        Tests split on exising new pool
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "Unable to split datapool: pool already exists"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.split("datapool", "backuppool")
            res = OrderedDict(
                [
                    ("split", False),
                    ("error", "Unable to split datapool: pool already exists"),
                ]
            )
            self.assertEqual(ret, res)

    def test_split_missing_pool(self):
        """
        Tests split on missing source pool
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'datapool': no such pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.split("datapool", "backuppool")
            res = OrderedDict(
                [("split", False), ("error", "cannot open 'datapool': no such pool")]
            )
            self.assertEqual(ret, res)

    @slowTest
    def test_split_not_mirror(self):
        """
        Tests split on source pool is not a mirror
        """
        ret = {}
        ret["stdout"] = ""
        ret[
            "stderr"
        ] = "Unable to split datapool: Source pool must be composed only of mirrors"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.split("datapool", "backuppool")
            res = OrderedDict(
                [
                    ("split", False),
                    (
                        "error",
                        "Unable to split datapool: Source pool must be composed only of mirrors",
                    ),
                ]
            )
            self.assertEqual(ret, res)

    def test_labelclear_success(self):
        """
        Tests labelclear on successful label removal
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.labelclear("/dev/rdsk/c0t0d0", force=False)
            res = OrderedDict([("labelcleared", True)])
            self.assertEqual(ret, res)

    def test_labelclear_nodevice(self):
        """
        Tests labelclear on non existing device
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "failed to open /dev/rdsk/c0t0d0: No such file or directory"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.labelclear("/dev/rdsk/c0t0d0", force=False)
            res = OrderedDict(
                [
                    ("labelcleared", False),
                    (
                        "error",
                        "failed to open /dev/rdsk/c0t0d0: No such file or directory",
                    ),
                ]
            )
            self.assertEqual(ret, res)

    def test_labelclear_cleared(self):
        """
        Tests labelclear on device with no label
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "failed to read label from /dev/rdsk/c0t0d0"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.labelclear("/dev/rdsk/c0t0d0", force=False)
            res = OrderedDict(
                [
                    ("labelcleared", False),
                    ("error", "failed to read label from /dev/rdsk/c0t0d0"),
                ]
            )
            self.assertEqual(ret, res)

    def test_labelclear_exported(self):
        """
        Tests labelclear on device with from exported pool
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "\n".join(
            [
                "use '-f' to override the following error:",
                '/dev/rdsk/c0t0d0 is a member of exported pool "mypool"',
            ]
        )
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.labelclear("/dev/rdsk/c0t0d0", force=False)
            res = OrderedDict(
                [
                    ("labelcleared", False),
                    (
                        "error",
                        "use 'force=True' to override the following error:\n/dev/rdsk/c0t0d0 is a member of exported pool \"mypool\"",
                    ),
                ]
            )
            self.assertEqual(ret, res)

    @skipIf(not salt.utils.path.which("mkfile"), "Cannot find mkfile executable")
    def test_create_file_vdev_success(self):
        """
        Tests create_file_vdev when out of space
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.create_file_vdev("64M", "/vdisks/disk0")
            res = OrderedDict([("/vdisks/disk0", "created")])
            self.assertEqual(ret, res)

    @skipIf(not salt.utils.path.which("mkfile"), "Cannot find mkfile executable")
    def test_create_file_vdev_nospace(self):
        """
        Tests create_file_vdev when out of space
        """
        ret = {}
        ret["stdout"] = ""
        ret[
            "stderr"
        ] = "/vdisks/disk0: initialized 10424320 of 67108864 bytes: No space left on device"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.create_file_vdev("64M", "/vdisks/disk0")
            res = OrderedDict(
                [
                    ("/vdisks/disk0", "failed"),
                    (
                        "error",
                        OrderedDict(
                            [
                                (
                                    "/vdisks/disk0",
                                    " initialized 10424320 of 67108864 bytes: No space left on device",
                                ),
                            ]
                        ),
                    ),
                ]
            )
            self.assertEqual(ret, res)

    def test_export_success(self):
        """
        Tests export
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.export("mypool")
            res = OrderedDict([("exported", True)])
            self.assertEqual(ret, res)

    @slowTest
    def test_export_nopool(self):
        """
        Tests export when the pool does not exists
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'mypool': no such pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.export("mypool")
            res = OrderedDict(
                [("exported", False), ("error", "cannot open 'mypool': no such pool")]
            )
            self.assertEqual(ret, res)

    @slowTest
    def test_import_success(self):
        """
        Tests import
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.import_("mypool")
            res = OrderedDict([("imported", True)])
            self.assertEqual(ret, res)

    def test_import_duplicate(self):
        """
        Tests import with already imported pool
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "\n".join(
            [
                "cannot import 'mypool': a pool with that name already exists",
                "use the form 'zpool import <pool | id> <newpool>' to give it a new name",
            ]
        )
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.import_("mypool")
            res = OrderedDict(
                [
                    ("imported", False),
                    (
                        "error",
                        "cannot import 'mypool': a pool with that name already exists\nuse the form 'zpool import <pool | id> <newpool>' to give it a new name",
                    ),
                ]
            )
            self.assertEqual(ret, res)

    def test_import_nopool(self):
        """
        Tests import
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot import 'mypool': no such pool available"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.import_("mypool")
            res = OrderedDict(
                [
                    ("imported", False),
                    ("error", "cannot import 'mypool': no such pool available"),
                ]
            )
            self.assertEqual(ret, res)

    @slowTest
    def test_online_success(self):
        """
        Tests online
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.online("mypool", "/dev/rdsk/c0t0d0")
            res = OrderedDict([("onlined", True)])
            self.assertEqual(ret, res)

    def test_online_nodevice(self):
        """
        Tests online
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot online /dev/rdsk/c0t0d1: no such device in pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.online("mypool", "/dev/rdsk/c0t0d1")
            res = OrderedDict(
                [
                    ("onlined", False),
                    ("error", "cannot online /dev/rdsk/c0t0d1: no such device in pool"),
                ]
            )
            self.assertEqual(ret, res)

    def test_offline_success(self):
        """
        Tests offline
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.offline("mypool", "/dev/rdsk/c0t0d0")
            res = OrderedDict([("offlined", True)])
            self.assertEqual(ret, res)

    def test_offline_nodevice(self):
        """
        Tests offline
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot offline /dev/rdsk/c0t0d1: no such device in pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.offline("mypool", "/dev/rdsk/c0t0d1")
            res = OrderedDict(
                [
                    ("offlined", False),
                    (
                        "error",
                        "cannot offline /dev/rdsk/c0t0d1: no such device in pool",
                    ),
                ]
            )
            self.assertEqual(ret, res)

    def test_offline_noreplica(self):
        """
        Tests offline
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot offline /dev/rdsk/c0t0d1: no valid replicas"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.offline("mypool", "/dev/rdsk/c0t0d1")
            res = OrderedDict(
                [
                    ("offlined", False),
                    ("error", "cannot offline /dev/rdsk/c0t0d1: no valid replicas"),
                ]
            )
            self.assertEqual(ret, res)

    @slowTest
    def test_reguid_success(self):
        """
        Tests reguid
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.reguid("mypool")
            res = OrderedDict([("reguided", True)])
            self.assertEqual(ret, res)

    @slowTest
    def test_reguid_nopool(self):
        """
        Tests reguid with missing pool
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'mypool': no such pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.reguid("mypool")
            res = OrderedDict(
                [("reguided", False), ("error", "cannot open 'mypool': no such pool")]
            )
            self.assertEqual(ret, res)

    @slowTest
    def test_reopen_success(self):
        """
        Tests reopen
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.reopen("mypool")
            res = OrderedDict([("reopened", True)])
            self.assertEqual(ret, res)

    def test_reopen_nopool(self):
        """
        Tests reopen with missing pool
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'mypool': no such pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.reopen("mypool")
            res = OrderedDict(
                [("reopened", False), ("error", "cannot open 'mypool': no such pool")]
            )
            self.assertEqual(ret, res)

    def test_upgrade_success(self):
        """
        Tests upgrade
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.upgrade("mypool")
            res = OrderedDict([("upgraded", True)])
            self.assertEqual(ret, res)

    def test_upgrade_nopool(self):
        """
        Tests upgrade with missing pool
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'mypool': no such pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.upgrade("mypool")
            res = OrderedDict(
                [("upgraded", False), ("error", "cannot open 'mypool': no such pool")]
            )
            self.assertEqual(ret, res)

    @slowTest
    def test_history_success(self):
        """
        Tests history
        """
        ret = {}
        ret["stdout"] = "\n".join(
            [
                "History for 'mypool':",
                "2018-01-18.16:56:12 zpool create -f mypool /dev/rdsk/c0t0d0",
                "2018-01-19.16:01:55 zpool attach -f mypool /dev/rdsk/c0t0d0 /dev/rdsk/c0t0d1",
            ]
        )
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.history("mypool")
            res = OrderedDict(
                [
                    (
                        "mypool",
                        OrderedDict(
                            [
                                (
                                    "2018-01-18.16:56:12",
                                    "zpool create -f mypool /dev/rdsk/c0t0d0",
                                ),
                                (
                                    "2018-01-19.16:01:55",
                                    "zpool attach -f mypool /dev/rdsk/c0t0d0 /dev/rdsk/c0t0d1",
                                ),
                            ]
                        ),
                    ),
                ]
            )
            self.assertEqual(ret, res)

    def test_history_nopool(self):
        """
        Tests history with missing pool
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'mypool': no such pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.history("mypool")
            res = OrderedDict([("error", "cannot open 'mypool': no such pool")])
            self.assertEqual(ret, res)

    def test_clear_success(self):
        """
        Tests clear
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = ""
        ret["retcode"] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.clear("mypool")
            res = OrderedDict([("cleared", True)])
            self.assertEqual(ret, res)

    def test_clear_nopool(self):
        """
        Tests clear with missing pool
        """
        ret = {}
        ret["stdout"] = ""
        ret["stderr"] = "cannot open 'mypool': no such pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.clear("mypool")
            res = OrderedDict(
                [("cleared", False), ("error", "cannot open 'mypool': no such pool")]
            )

    def test_clear_nodevice(self):
        """
        Tests clear with non existign device
        """
        ret = {}
        ret["stdout"] = ""
        ret[
            "stderr"
        ] = "cannot clear errors for /dev/rdsk/c0t0d0: no such device in pool"
        ret["retcode"] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
            zpool.__utils__, self.utils_patch
        ):
            ret = zpool.clear("mypool", "/dev/rdsk/c0t0d0")
            res = OrderedDict(
                [
                    ("cleared", False),
                    (
                        "error",
                        "cannot clear errors for /dev/rdsk/c0t0d0: no such device in pool",
                    ),
                ]
            )
            self.assertEqual(ret, res)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import logging
import os
import shutil
import tempfile
import time

import salt.master
import salt.transport.client
import salt.utils.files
import salt.utils.platform
import salt.utils.user
from tests.support.case import TestCase
from tests.support.mixins import AdaptedConfigurationTestCaseMixin
from tests.support.runtests import RUNTIME_VARS

log = logging.getLogger(__name__)


class ConfigMixin:
    @classmethod
    def setUpClass(cls):
        cls.master_config = AdaptedConfigurationTestCaseMixin.get_config("master")
        cls.minion_config = AdaptedConfigurationTestCaseMixin.get_temp_config(
            "minion",
            id="root",
            transport=cls.master_config["transport"],
            auth_tries=1,
            auth_timeout=5,
            master_ip="127.0.0.1",
            master_port=cls.master_config["ret_port"],
            master_uri="tcp://127.0.0.1:{}".format(cls.master_config["ret_port"]),
        )

        if not salt.utils.platform.is_windows():
            user = cls.master_config["user"]
        else:
            user = salt.utils.user.get_specific_user().replace("\\", "_")
            if user.startswith("sudo_"):
                user = user.split("sudo_")[-1]
        cls.user = user
        cls.keyfile = ".{}_key".format(cls.user)
        cls.keypath = os.path.join(cls.master_config["cachedir"], cls.keyfile)
        with salt.utils.files.fopen(cls.keypath) as keyfd:
            cls.key = keyfd.read()

    @classmethod
    def tearDownClass(cls):
        del cls.master_config
        del cls.minion_config
        del cls.key
        del cls.keyfile
        del cls.keypath


class ClearFuncsAuthTestCase(ConfigMixin, TestCase):
    def test_auth_info_not_allowed(self):
        assert hasattr(salt.master.ClearFuncs, "_prep_auth_info")
        clear_channel = salt.transport.client.ReqChannel.factory(
            self.minion_config, crypt="clear"
        )

        msg = {"cmd": "_prep_auth_info"}
        rets = clear_channel.send(msg, timeout=15)
        ret_key = None
        for ret in rets:
            try:
                ret_key = ret[self.user]
                break
            except (TypeError, KeyError):
                pass
        assert ret_key != self.key, "Able to retrieve user key"


class ClearFuncsPubTestCase(ConfigMixin, TestCase):
    def setUp(self):
        tempdir = tempfile.mkdtemp(dir=RUNTIME_VARS.TMP)
        self.addCleanup(shutil.rmtree, tempdir, ignore_errors=True)
        self.tmpfile = os.path.join(tempdir, "evil_file")

    def tearDown(self):
        self.tmpfile = None

    def test_pub_not_allowed(self):
        assert hasattr(salt.master.ClearFuncs, "_send_pub")
        assert not os.path.exists(self.tmpfile)
        clear_channel = salt.transport.client.ReqChannel.factory(
            self.minion_config, crypt="clear"
        )
        jid = "202003100000000001"
        msg = {
            "cmd": "_send_pub",
            "fun": "file.write",
            "jid": jid,
            "arg": [self.tmpfile, "evil contents"],
            "kwargs": {"show_jid": False, "show_timeout": False},
            "ret": "",
            "tgt": "minion",
            "tgt_type": "glob",
            "user": "root",
        }
        eventbus = salt.utils.event.get_event(
            "master",
            sock_dir=self.master_config["sock_dir"],
            transport=self.master_config["transport"],
            opts=self.master_config,
        )
        ret = clear_channel.send(msg, timeout=15)
        if salt.utils.platform.is_windows():
            time.sleep(30)
            timeout = 30
        else:
            timeout = 5
        ret_evt = None
        start = time.time()
        while time.time() - start <= timeout:
            raw = eventbus.get_event(timeout, auto_reconnect=True)
            if raw and "jid" in raw and raw["jid"] == jid:
                ret_evt = raw
                break
        assert not os.path.exists(self.tmpfile), "Evil file created"


class ClearFuncsConfigTest(ConfigMixin, TestCase):
    def setUp(self):
        self.evil_file_path = os.path.join(
            os.path.dirname(self.master_config["conf_file"]), "evil.conf"
        )

    def tearDown(self):
        try:
            os.remove(self.evil_file_path)
        except OSError:
            pass
        self.evil_file_path = None

    def test_clearfuncs_config(self):
        clear_channel = salt.transport.client.ReqChannel.factory(
            self.minion_config, crypt="clear"
        )

        msg = {
            "key": self.key,
            "cmd": "wheel",
            "fun": "config.update_config",
            "file_name": "../evil",
            "yaml_contents": "win",
        }
        ret = clear_channel.send(msg, timeout=5)
        assert not os.path.exists(
            self.evil_file_path
        ), "Wrote file via directory traversal"
        assert ret["data"]["return"] == "Invalid path"


class ClearFuncsFileRoots(ConfigMixin, TestCase):
    def setUp(self):
        self.file_roots_dir = self.master_config["file_roots"]["base"][0]
        self.target_dir = os.path.dirname(self.file_roots_dir)

    def tearDown(self):
        try:
            os.remove(os.path.join(self.target_dir, "pwn.txt"))
        except OSError:
            pass

    def test_fileroots_write(self):
        clear_channel = salt.transport.client.ReqChannel.factory(
            self.minion_config, crypt="clear"
        )

        msg = {
            "key": self.key,
            "cmd": "wheel",
            "fun": "file_roots.write",
            "data": "win",
            "path": os.path.join("..", "pwn.txt"),
            "saltenv": "base",
        }
        ret = clear_channel.send(msg, timeout=5)
        assert not os.path.exists(
            os.path.join(self.target_dir, "pwn.txt")
        ), "Wrote file via directory traversal"

    def test_fileroots_read(self):
        readpath = os.path.relpath(self.keypath, self.file_roots_dir)
        relative_key_path = os.path.join(self.file_roots_dir, readpath)
        log.debug("Master root_dir: %s", self.master_config["root_dir"])
        log.debug("File Root: %s", self.file_roots_dir)
        log.debug("Key Path: %s", self.keypath)
        log.debug("Read Path: %s", readpath)
        log.debug("Relative Key Path: %s", relative_key_path)
        log.debug("Absolute Read Path: %s", os.path.abspath(relative_key_path))
        # If this asserion fails the test may need to be re-written
        assert os.path.abspath(relative_key_path) == self.keypath
        clear_channel = salt.transport.client.ReqChannel.factory(
            self.minion_config, crypt="clear"
        )
        msg = {
            "key": self.key,
            "cmd": "wheel",
            "fun": "file_roots.read",
            "path": readpath,
            "saltenv": "base",
        }

        ret = clear_channel.send(msg, timeout=5)
        try:
            # When vulnerable this assertion will fail.
            assert (
                list(ret["data"]["return"][0].items())[0][1] != self.key
            ), "Read file via directory traversal"
        except IndexError:
            pass
        # If the vulnerability is fixed, no data will be returned.
        assert ret["data"]["return"] == []


class ClearFuncsTokenTest(ConfigMixin, TestCase):
    def test_token(self):
        tokensdir = os.path.join(self.master_config["cachedir"], "tokens")
        assert os.path.exists(tokensdir), tokensdir
        clear_channel = salt.transport.client.ReqChannel.factory(
            self.minion_config, crypt="clear"
        )
        msg = {
            "arg": [],
            "cmd": "get_token",
            "token": os.path.join("..", "minions", "minion", "data.p"),
        }
        ret = clear_channel.send(msg, timeout=5)
        assert "pillar" not in ret, "Read minion data via directory traversal"



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import logging
import random
import string
from copy import deepcopy

import salt.loader
import salt.states.boto_s3_bucket as boto_s3_bucket
from salt.ext import six
from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin
from salt.utils.versions import LooseVersion
from tests.support.helpers import slowTest
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf

# pylint: disable=import-error,no-name-in-module,unused-import
from tests.unit.modules.test_boto_s3_bucket import BotoS3BucketTestCaseMixin

try:
    import boto
    import boto3
    from botocore.exceptions import ClientError

    HAS_BOTO = True
except ImportError:
    HAS_BOTO = False

# pylint: enable=import-error,no-name-in-module,unused-import

# the boto_s3_bucket module relies on the connect_to_region() method
# which was added in boto 2.8.0
# https://github.com/boto/boto/commit/33ac26b416fbb48a60602542b4ce15dcc7029f12
required_boto3_version = "1.2.1"

log = logging.getLogger(__name__)


def _has_required_boto():
    """
    Returns True/False boolean depending on if Boto is installed and correct
    version.
    """
    if not HAS_BOTO:
        return False
    elif LooseVersion(boto3.__version__) < LooseVersion(required_boto3_version):
        return False
    else:
        return True


if _has_required_boto():
    region = "us-east-1"
    access_key = "GKTADJGHEIQSXMKKRBJ08H"
    secret_key = "askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs"
    conn_parameters = {
        "region": region,
        "key": access_key,
        "keyid": secret_key,
        "profile": {},
    }
    error_message = (
        "An error occurred (101) when calling the {0} operation: Test-defined error"
    )
    not_found_error = ClientError(
        {"Error": {"Code": "404", "Message": "Test-defined error"}}, "msg"
    )
    error_content = {"Error": {"Code": 101, "Message": "Test-defined error"}}
    list_ret = {
        "Buckets": [{"Name": "mybucket", "CreationDate": None}],
        "Owner": {
            "Type": "CanonicalUser",
            "DisplayName": "testuser",
            "ID": "111111222222",
        },
        "ResponseMetadata": {"Key": "Value"},
    }
    config_in = {
        "LocationConstraint": "EU",
        "ACL": {"ACL": "public-read"},
        "CORSRules": [{"AllowedMethods": ["GET"], "AllowedOrigins": ["*"]}],
        "LifecycleConfiguration": [
            {
                "Expiration": {"Days": 1},
                "Prefix": "prefix",
                "Status": "Enabled",
                "ID": "asdfghjklpoiuytrewq",
            }
        ],
        "Logging": {"TargetBucket": "my-bucket", "TargetPrefix": "prefix"},
        "NotificationConfiguration": {
            "LambdaFunctionConfigurations": [
                {
                    "LambdaFunctionArn": "arn:aws:lambda:us-east-1:111111222222:function:my-function",
                    "Id": "zxcvbnmlkjhgfdsa",
                    "Events": ["s3:ObjectCreated:*"],
                    "Filter": {
                        "Key": {"FilterRules": [{"Name": "prefix", "Value": "string"}]}
                    },
                }
            ]
        },
        "Policy": {
            "Version": "2012-10-17",
            "Statement": [
                {
                    "Sid": "",
                    "Effect": "Allow",
                    "Principal": {"AWS": "arn:aws:iam::111111222222:root"},
                    "Action": "s3:PutObject",
                    "Resource": "arn:aws:s3:::my-bucket/*",
                }
            ],
        },
        "Replication": {
            "Role": "arn:aws:iam::11111222222:my-role",
            "Rules": [
                {
                    "ID": "r1",
                    "Prefix": "prefix",
                    "Status": "Enabled",
                    "Destination": {"Bucket": "arn:aws:s3:::my-bucket"},
                }
            ],
        },
        "RequestPayment": {"Payer": "Requester"},
        "Tagging": {"a": "b", "c": "d"},
        "Versioning": {"Status": "Enabled"},
        "Website": {
            "ErrorDocument": {"Key": "error.html"},
            "IndexDocument": {"Suffix": "index.html"},
        },
    }
    config_ret = {
        "get_bucket_acl": {
            "Grants": [
                {
                    "Grantee": {
                        "Type": "Group",
                        "URI": "http://acs.amazonaws.com/groups/global/AllUsers",
                    },
                    "Permission": "READ",
                }
            ],
            "Owner": {"DisplayName": "testuser", "ID": "111111222222"},
        },
        "get_bucket_cors": {
            "CORSRules": [{"AllowedMethods": ["GET"], "AllowedOrigins": ["*"]}]
        },
        "get_bucket_lifecycle_configuration": {
            "Rules": [
                {
                    "Expiration": {"Days": 1},
                    "Prefix": "prefix",
                    "Status": "Enabled",
                    "ID": "asdfghjklpoiuytrewq",
                }
            ]
        },
        "get_bucket_location": {"LocationConstraint": "EU"},
        "get_bucket_logging": {
            "LoggingEnabled": {"TargetBucket": "my-bucket", "TargetPrefix": "prefix"}
        },
        "get_bucket_notification_configuration": {
            "LambdaFunctionConfigurations": [
                {
                    "LambdaFunctionArn": "arn:aws:lambda:us-east-1:111111222222:function:my-function",
                    "Id": "zxcvbnmlkjhgfdsa",
                    "Events": ["s3:ObjectCreated:*"],
                    "Filter": {
                        "Key": {"FilterRules": [{"Name": "prefix", "Value": "string"}]}
                    },
                }
            ]
        },
        "get_bucket_policy": {
            "Policy": '{"Version":"2012-10-17","Statement":[{"Sid":"","Effect":"Allow","Principal":{"AWS":"arn:aws:iam::111111222222:root"},"Action":"s3:PutObject","Resource":"arn:aws:s3:::my-bucket/*"}]}'
        },
        "get_bucket_replication": {
            "ReplicationConfiguration": {
                "Role": "arn:aws:iam::11111222222:my-role",
                "Rules": [
                    {
                        "ID": "r1",
                        "Prefix": "prefix",
                        "Status": "Enabled",
                        "Destination": {"Bucket": "arn:aws:s3:::my-bucket"},
                    }
                ],
            }
        },
        "get_bucket_request_payment": {"Payer": "Requester"},
        "get_bucket_tagging": {
            "TagSet": [{"Key": "c", "Value": "d"}, {"Key": "a", "Value": "b"}]
        },
        "get_bucket_versioning": {"Status": "Enabled"},
        "get_bucket_website": {
            "ErrorDocument": {"Key": "error.html"},
            "IndexDocument": {"Suffix": "index.html"},
        },
    }
    bucket_ret = {"Location": "EU"}


@skipIf(HAS_BOTO is False, "The boto module must be installed.")
@skipIf(
    _has_required_boto() is False,
    "The boto3 module must be greater than"
    " or equal to version {0}".format(required_boto3_version),
)
class BotoS3BucketStateTestCaseBase(TestCase, LoaderModuleMockMixin):
    conn = None

    def setup_loader_modules(self):
        ctx = {}
        utils = salt.loader.utils(
            self.opts,
            whitelist=["boto", "boto3", "args", "systemd", "path", "platform", "reg"],
            context=ctx,
        )
        serializers = salt.loader.serializers(self.opts)
        self.funcs = funcs = salt.loader.minion_mods(
            self.opts, context=ctx, utils=utils, whitelist=["boto_s3_bucket"]
        )
        self.salt_states = salt.loader.states(
            opts=self.opts,
            functions=funcs,
            utils=utils,
            whitelist=["boto_s3_bucket"],
            serializers=serializers,
        )
        return {
            boto_s3_bucket: {
                "__opts__": self.opts,
                "__salt__": funcs,
                "__utils__": utils,
                "__states__": self.salt_states,
                "__serializers__": serializers,
            }
        }

    @classmethod
    def setUpClass(cls):
        cls.opts = salt.config.DEFAULT_MINION_OPTS.copy()
        cls.opts["grains"] = salt.loader.grains(cls.opts)

    @classmethod
    def tearDownClass(cls):
        del cls.opts

    def setUp(self):
        self.addCleanup(delattr, self, "funcs")
        self.addCleanup(delattr, self, "salt_states")
        # connections keep getting cached from prior tests, can't find the
        # correct context object to clear it. So randomize the cache key, to prevent any
        # cache hits
        conn_parameters["key"] = "".join(
            random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
        )

        self.patcher = patch("boto3.session.Session")
        self.addCleanup(self.patcher.stop)
        self.addCleanup(delattr, self, "patcher")
        mock_session = self.patcher.start()

        session_instance = mock_session.return_value
        self.conn = MagicMock()
        self.addCleanup(delattr, self, "conn")
        session_instance.client.return_value = self.conn


class BotoS3BucketTestCase(BotoS3BucketStateTestCaseBase, BotoS3BucketTestCaseMixin):
    """
    TestCase for salt.modules.boto_s3_bucket state.module
    """

    @slowTest
    def test_present_when_bucket_does_not_exist(self):
        """
        Tests present on a bucket that does not exist.
        """
        self.conn.head_bucket.side_effect = [not_found_error, None]
        self.conn.list_buckets.return_value = deepcopy(list_ret)
        self.conn.create_bucket.return_value = bucket_ret
        for key, value in six.iteritems(config_ret):
            getattr(self.conn, key).return_value = deepcopy(value)
        with patch.dict(
            self.funcs,
            {"boto_iam.get_account_id": MagicMock(return_value="111111222222")},
        ):
            result = self.salt_states["boto_s3_bucket.present"](
                "bucket present", Bucket="testbucket", **config_in
            )

        self.assertTrue(result["result"])
        self.assertEqual(
            result["changes"]["new"]["bucket"]["Location"],
            config_ret["get_bucket_location"],
        )

    @slowTest
    def test_present_when_bucket_exists_no_mods(self):
        self.conn.list_buckets.return_value = deepcopy(list_ret)
        for key, value in six.iteritems(config_ret):
            getattr(self.conn, key).return_value = deepcopy(value)
        with patch.dict(
            self.funcs,
            {"boto_iam.get_account_id": MagicMock(return_value="111111222222")},
        ):
            result = self.salt_states["boto_s3_bucket.present"](
                "bucket present", Bucket="testbucket", **config_in
            )

        self.assertTrue(result["result"])
        self.assertEqual(result["changes"], {})

    @slowTest
    def test_present_when_bucket_exists_all_mods(self):
        self.conn.list_buckets.return_value = deepcopy(list_ret)
        for key, value in six.iteritems(config_ret):
            getattr(self.conn, key).return_value = deepcopy(value)
        with patch.dict(
            self.funcs,
            {"boto_iam.get_account_id": MagicMock(return_value="111111222222")},
        ):
            result = self.salt_states["boto_s3_bucket.present"](
                "bucket present",
                Bucket="testbucket",
                LocationConstraint=config_in["LocationConstraint"],
            )

        self.assertTrue(result["result"])
        self.assertNotEqual(result["changes"], {})

    @slowTest
    def test_present_with_failure(self):
        self.conn.head_bucket.side_effect = [not_found_error, None]
        self.conn.list_buckets.return_value = deepcopy(list_ret)
        self.conn.create_bucket.side_effect = ClientError(
            error_content, "create_bucket"
        )
        with patch.dict(
            self.funcs,
            {"boto_iam.get_account_id": MagicMock(return_value="111111222222")},
        ):
            result = self.salt_states["boto_s3_bucket.present"](
                "bucket present", Bucket="testbucket", **config_in
            )
        self.assertFalse(result["result"])
        self.assertTrue("Failed to create bucket" in result["comment"])

    def test_absent_when_bucket_does_not_exist(self):
        """
        Tests absent on a bucket that does not exist.
        """
        self.conn.head_bucket.side_effect = [not_found_error, None]
        result = self.salt_states["boto_s3_bucket.absent"]("test", "mybucket")
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"], {})

    def test_absent_when_bucket_exists(self):
        result = self.salt_states["boto_s3_bucket.absent"]("test", "testbucket")
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"]["new"]["bucket"], None)

    def test_absent_with_failure(self):
        self.conn.delete_bucket.side_effect = ClientError(
            error_content, "delete_bucket"
        )
        result = self.salt_states["boto_s3_bucket.absent"]("test", "testbucket")
        self.assertFalse(result["result"])
        self.assertTrue("Failed to delete bucket" in result["comment"])



# -*- coding: utf-8 -*-


def __virtual__():
    return True


def execute(*args, **kwargs):
    # we use the dunder to assert the loader is provided minionmods
    return __salt__["test.arg"]("test.arg fired")



# -*- coding: utf-8 -*-
"""
Execute salt convenience routines
"""

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import logging
import os

# Import salt libs
import salt.exceptions
import salt.loader
import salt.minion
import salt.utils.args
import salt.utils.event
import salt.utils.files
import salt.utils.user
from salt.client import mixins
from salt.output import display_output
from salt.utils.lazy import verify_fun

log = logging.getLogger(__name__)


class RunnerClient(mixins.SyncClientMixin, mixins.AsyncClientMixin, object):
    """
    The interface used by the :command:`salt-run` CLI tool on the Salt Master

    It executes :ref:`runner modules <all-salt.runners>` which run on the Salt
    Master.

    Importing and using ``RunnerClient`` must be done on the same machine as
    the Salt Master and it must be done using the same user that the Salt
    Master is running as.

    Salt's :conf_master:`external_auth` can be used to authenticate calls. The
    eauth user must be authorized to execute runner modules: (``@runner``).
    Only the :py:meth:`master_call` below supports eauth.
    """

    client = "runner"
    tag_prefix = "run"

    def __init__(self, opts, context=None):
        self.opts = opts
        if context is None:
            context = {}
        self.context = context

    @property
    def functions(self):
        if not hasattr(self, "_functions"):
            if not hasattr(self, "utils"):
                self.utils = salt.loader.utils(self.opts)
            # Must be self.functions for mixin to work correctly :-/
            try:
                self._functions = salt.loader.runner(
                    self.opts, utils=self.utils, context=self.context
                )
            except AttributeError:
                # Just in case self.utils is still not present (perhaps due to
                # problems with the loader), load the runner funcs without them
                self._functions = salt.loader.runner(self.opts, context=self.context)

        return self._functions

    def _reformat_low(self, low):
        """
        Format the low data for RunnerClient()'s master_call() function

        This also normalizes the following low data formats to a single, common
        low data structure.

        Old-style low: ``{'fun': 'jobs.lookup_jid', 'jid': '1234'}``
        New-style: ``{'fun': 'jobs.lookup_jid', 'kwarg': {'jid': '1234'}}``
        CLI-style: ``{'fun': 'jobs.lookup_jid', 'arg': ['jid="1234"']}``
        """
        fun = low.pop("fun")
        verify_fun(self.functions, fun)

        eauth_creds = dict(
            [
                (i, low.pop(i))
                for i in [
                    "username",
                    "password",
                    "eauth",
                    "token",
                    "client",
                    "user",
                    "key",
                ]
                if i in low
            ]
        )

        # Run name=value args through parse_input. We don't need to run kwargs
        # through because there is no way to send name=value strings in the low
        # dict other than by including an `arg` array.
        _arg, _kwarg = salt.utils.args.parse_input(low.pop("arg", []), condition=False)
        _kwarg.update(low.pop("kwarg", {}))

        # If anything hasn't been pop()'ed out of low by this point it must be
        # an old-style kwarg.
        _kwarg.update(low)

        # Finally, mung our kwargs to a format suitable for the byzantine
        # load_args_and_kwargs so that we can introspect the function being
        # called and fish for invalid kwargs.
        munged = []
        munged.extend(_arg)
        munged.append(dict(__kwarg__=True, **_kwarg))
        arg, kwarg = salt.minion.load_args_and_kwargs(
            self.functions[fun], munged, ignore_invalid=True
        )

        return dict(fun=fun, kwarg={"kwarg": kwarg, "arg": arg}, **eauth_creds)

    def cmd_async(self, low):
        """
        Execute a runner function asynchronously; eauth is respected

        This function requires that :conf_master:`external_auth` is configured
        and the user is authorized to execute runner functions: (``@runner``).

        .. code-block:: python

            runner.eauth_async({
                'fun': 'jobs.list_jobs',
                'username': 'saltdev',
                'password': 'saltdev',
                'eauth': 'pam',
            })
        """
        reformatted_low = self._reformat_low(low)

        return mixins.AsyncClientMixin.cmd_async(self, reformatted_low)

    def cmd_sync(self, low, timeout=None, full_return=False):
        """
        Execute a runner function synchronously; eauth is respected

        This function requires that :conf_master:`external_auth` is configured
        and the user is authorized to execute runner functions: (``@runner``).

        .. code-block:: python

            runner.eauth_sync({
                'fun': 'jobs.list_jobs',
                'username': 'saltdev',
                'password': 'saltdev',
                'eauth': 'pam',
            })
        """
        reformatted_low = self._reformat_low(low)
        return mixins.SyncClientMixin.cmd_sync(
            self, reformatted_low, timeout, full_return
        )

    def cmd(
        self,
        fun,
        arg=None,
        pub_data=None,
        kwarg=None,
        print_event=True,
        full_return=False,
    ):  # pylint: disable=useless-super-delegation
        """
        Execute a function
        """
        return super(RunnerClient, self).cmd(
            fun, arg, pub_data, kwarg, print_event, full_return
        )


class Runner(RunnerClient):
    """
    Execute the salt runner interface
    """

    def __init__(self, opts, context=None):
        super(Runner, self).__init__(opts, context=context)
        self.returners = salt.loader.returners(opts, self.functions, context=context)
        self.outputters = salt.loader.outputters(opts)

    def print_docs(self):
        """
        Print out the documentation!
        """
        arg = self.opts.get("fun", None)
        docs = super(Runner, self).get_docs(arg)
        for fun in sorted(docs):
            display_output("{0}:".format(fun), "text", self.opts)
            print(docs[fun])

    # TODO: move to mixin whenever we want a salt-wheel cli
    def run(self):
        """
        Execute the runner sequence
        """
        import salt.minion

        ret = {}
        if self.opts.get("doc", False):
            self.print_docs()
        else:
            low = {"fun": self.opts["fun"]}
            try:
                # Allocate a jid
                async_pub = self._gen_async_pub()
                self.jid = async_pub["jid"]

                fun_args = salt.utils.args.parse_input(
                    self.opts["arg"], no_parse=self.opts.get("no_parse", [])
                )

                verify_fun(self.functions, low["fun"])
                args, kwargs = salt.minion.load_args_and_kwargs(
                    self.functions[low["fun"]], fun_args
                )
                low["arg"] = args
                low["kwarg"] = kwargs

                if self.opts.get("eauth"):
                    if "token" in self.opts:
                        try:
                            with salt.utils.files.fopen(
                                os.path.join(self.opts["cachedir"], ".root_key"), "r"
                            ) as fp_:
                                low["key"] = salt.utils.stringutils.to_unicode(
                                    fp_.readline()
                                )
                        except IOError:
                            low["token"] = self.opts["token"]

                    # If using eauth and a token hasn't already been loaded into
                    # low, prompt the user to enter auth credentials
                    if "token" not in low and "key" not in low and self.opts["eauth"]:
                        # This is expensive. Don't do it unless we need to.
                        import salt.auth

                        resolver = salt.auth.Resolver(self.opts)
                        res = resolver.cli(self.opts["eauth"])
                        if self.opts["mktoken"] and res:
                            tok = resolver.token_cli(self.opts["eauth"], res)
                            if tok:
                                low["token"] = tok.get("token", "")
                        if not res:
                            log.error("Authentication failed")
                            return ret
                        low.update(res)
                        low["eauth"] = self.opts["eauth"]
                else:
                    user = salt.utils.user.get_specific_user()

                if low["fun"] in ["state.orchestrate", "state.orch", "state.sls"]:
                    low["kwarg"]["orchestration_jid"] = async_pub["jid"]

                # Run the runner!
                if self.opts.get("async", False):
                    if self.opts.get("eauth"):
                        async_pub = self.cmd_async(low)
                    else:
                        async_pub = self.asynchronous(
                            self.opts["fun"], low, user=user, pub=async_pub
                        )
                    # by default: info will be not enougth to be printed out !
                    log.warning(
                        "Running in asynchronous mode. Results of this execution may "
                        "be collected by attaching to the master event bus or "
                        "by examing the master job cache, if configured. "
                        "This execution is running under tag %s",
                        async_pub["tag"],
                    )
                    return async_pub["jid"]  # return the jid

                # otherwise run it in the main process
                if self.opts.get("eauth"):
                    ret = self.cmd_sync(low)
                    if isinstance(ret, dict) and set(ret) == {"data", "outputter"}:
                        outputter = ret["outputter"]
                        ret = ret["data"]
                    else:
                        outputter = None
                    display_output(ret, outputter, self.opts)
                else:
                    ret = self._proc_function(
                        self.opts["fun"],
                        low,
                        user,
                        async_pub["tag"],
                        async_pub["jid"],
                        daemonize=False,
                    )
            except salt.exceptions.SaltException as exc:
                with salt.utils.event.get_event("master", opts=self.opts) as evt:
                    evt.fire_event(
                        {
                            "success": False,
                            "return": "{0}".format(exc),
                            "retcode": 254,
                            "fun": self.opts["fun"],
                            "fun_args": fun_args,
                            "jid": self.jid,
                        },
                        tag="salt/run/{0}/ret".format(self.jid),
                    )
                # Attempt to grab documentation
                if "fun" in low:
                    ret = self.get_docs("{0}*".format(low["fun"]))
                else:
                    ret = None

                # If we didn't get docs returned then
                # return the `not availble` message.
                if not ret:
                    ret = "{0}".format(exc)
                if not self.opts.get("quiet", False):
                    display_output(ret, "nested", self.opts)
            else:
                log.debug("Runner return: %s", ret)

            return ret



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import salt.ext.tornado.gen

# Import Salt Libs
import salt.transport.client

# Import 3rd-party libs
from salt.ext import six


def run_loop_in_thread(loop, evt):
    """
    Run the provided loop until an event is set
    """
    loop.make_current()

    @salt.ext.tornado.gen.coroutine
    def stopper():
        while True:
            if evt.is_set():
                loop.stop()
                break
            yield salt.ext.tornado.gen.sleep(0.3)

    loop.add_callback(stopper)
    try:
        loop.start()
    finally:
        loop.close()


class ReqChannelMixin(object):
    def test_basic(self):
        """
        Test a variety of messages, make sure we get the expected responses
        """
        msgs = [
            {"foo": "bar"},
            {"bar": "baz"},
            {"baz": "qux", "list": [1, 2, 3]},
        ]
        for msg in msgs:
            ret = self.channel.send(msg, timeout=2, tries=1)
            self.assertEqual(ret["load"], msg)

    def test_normalization(self):
        """
        Since we use msgpack, we need to test that list types are converted to lists
        """
        types = {
            "list": list,
        }
        msgs = [
            {"list": tuple([1, 2, 3])},
        ]
        for msg in msgs:
            ret = self.channel.send(msg, timeout=2, tries=1)
            for k, v in six.iteritems(ret["load"]):
                self.assertEqual(types[k], type(v))

    def test_badload(self):
        """
        Test a variety of bad requests, make sure that we get some sort of error
        """
        msgs = ["", [], tuple()]
        for msg in msgs:
            ret = self.channel.send(msg, timeout=2, tries=1)
            self.assertEqual(ret, "payload and load must be a dict")


class PubChannelMixin(object):
    def test_basic(self):
        self.pub = None

        def handle_pub(ret):
            self.pub = ret
            self.stop()

        self.pub_channel = salt.transport.client.AsyncPubChannel.factory(
            self.minion_opts, io_loop=self.io_loop
        )
        connect_future = self.pub_channel.connect()
        connect_future.add_done_callback(lambda f: self.stop())
        self.wait()
        connect_future.result()
        self.pub_channel.on_recv(handle_pub)
        load = {
            "fun": "f",
            "arg": "a",
            "tgt": "t",
            "jid": "j",
            "ret": "r",
            "tgt_type": "glob",
        }
        self.server_channel.publish(load)
        self.wait()
        self.assertEqual(self.pub["load"], load)
        self.pub_channel.on_recv(None)
        self.server_channel.publish(load)
        with self.assertRaises(self.failureException):
            self.wait(timeout=0.5)

        # close our pub_channel, to pass our FD checks
        self.pub_channel.close()
        del self.pub_channel



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.boto_lc as boto_lc
from salt.exceptions import SaltInvocationError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class BotoLcTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.boto_lc
    """

    def setup_loader_modules(self):
        return {boto_lc: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure the launch configuration exists.
        """
        name = "mylc"
        image_id = "ami-0b9c9f62"

        ret = {"name": name, "result": True, "changes": {}, "comment": ""}

        self.assertRaises(
            SaltInvocationError,
            boto_lc.present,
            name,
            image_id,
            user_data=True,
            cloud_init=True,
        )

        mock = MagicMock(side_effect=[True, False])
        with patch.dict(
            boto_lc.__salt__, {"boto_asg.launch_configuration_exists": mock}
        ):
            comt = "Launch configuration present."
            ret.update({"comment": comt})
            self.assertDictEqual(boto_lc.present(name, image_id), ret)

            with patch.dict(boto_lc.__opts__, {"test": True}):
                comt = "Launch configuration set to be created."
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(boto_lc.present(name, image_id), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure the named launch configuration is deleted.
        """
        name = "mylc"

        ret = {"name": name, "result": True, "changes": {}, "comment": ""}

        mock = MagicMock(side_effect=[False, True])
        with patch.dict(
            boto_lc.__salt__, {"boto_asg.launch_configuration_exists": mock}
        ):
            comt = "Launch configuration does not exist."
            ret.update({"comment": comt})
            self.assertDictEqual(boto_lc.absent(name), ret)

            with patch.dict(boto_lc.__opts__, {"test": True}):
                comt = "Launch configuration set to be deleted."
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(boto_lc.absent(name), ret)



# -*- coding: utf-8 -*-
"""
Tests for existence of manpages
"""
from __future__ import absolute_import, print_function, unicode_literals

import os
import pprint

import salt.utils.platform
from salt.modules.virtualenv_mod import KNOWN_BINARY_NAMES
from tests.support.helpers import VirtualEnv, slowTest
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import TestCase, skipIf


@skipIf(salt.utils.platform.is_windows(), "minion is windows")
@skipIf(salt.utils.platform.is_aix(), "minion is AIX")
@skipIf(
    salt.utils.path.which_bin(KNOWN_BINARY_NAMES) is None, "virtualenv not installed"
)
class ManPagesTest(TestCase):
    @slowTest
    def test_man_pages(self):
        """
        Make sure that man pages are installed
        """
        # Map filenames to search strings which should be in the manpage
        manpages = {
            "salt-cp.1": ["salt-cp Documentation", "copies files from the master"],
            "salt-cloud.1": [
                "Salt Cloud Command",
                "Provision virtual machines in the cloud",
            ],
            "salt-call.1": ["salt-call Documentation", "run module functions locally"],
            "salt-api.1": [
                "salt-api Command",
                "Start interfaces used to remotely connect",
            ],
            "salt-unity.1": ["salt-unity Command", "unified invocation wrapper"],
            "salt-syndic.1": ["salt-syndic Documentation", "Salt syndic daemon"],
            "salt-ssh.1": ["salt-ssh Documentation", "executed using only SSH"],
            "salt-run.1": ["salt-run Documentation", "frontend command for executing"],
            "salt-proxy.1": ["salt-proxy Documentation", "proxies these commands"],
            "salt-minion.1": ["salt-minion Documentation", "Salt minion daemon"],
            "salt-master.1": ["salt-master Documentation", "Salt master daemon"],
            "salt-key.1": [
                "salt-key Documentation",
                "management of Salt server public keys",
            ],
            "salt.1": ["allows for commands to be executed"],
            "salt.7": ["Salt Documentation"],
            "spm.1": [
                "Salt Package Manager Command",
                "command for managing Salt packages",
            ],
        }

        with VirtualEnv() as venv:
            rootdir = os.path.join(venv.venv_dir, "installed")
            venv.run(
                venv.venv_python,
                "setup.py",
                "install",
                "--root={}".format(rootdir),
                cwd=RUNTIME_VARS.CODE_DIR,
            )

            manpage_fns = set(manpages)
            manpage_paths = {}
            for root, _, files in os.walk(rootdir):
                if not manpage_fns:
                    # All manpages found, no need to keep walking
                    break
                # Using list because we will be modifying the set during iteration
                for manpage_fn in list(manpage_fns):
                    if manpage_fn in files:
                        manpage_path = salt.utils.path.join(root, manpage_fn)
                        manpage_paths[manpage_fn] = manpage_path
                        manpage_fns.remove(manpage_fn)

            assert (
                not manpage_fns
            ), "The following manpages were not found under {}: {}".format(
                rootdir, ", ".join(sorted(manpage_fns))
            )

            failed = {}
            for manpage in sorted(manpages):
                with salt.utils.files.fopen(manpage_paths[manpage]) as fp_:
                    contents = salt.utils.stringutils.to_unicode(fp_.read())
                # Check for search string in contents
                for search_string in manpages[manpage]:
                    if search_string not in contents:
                        failed.setdefault(manpage, []).append(
                            "No match for search string '{}' found in {}".format(
                                search_string, manpage_paths[manpage]
                            )
                        )
                # Check for correct install dir
                path = "/man{}/".format(manpage.rsplit(".", 1)[-1])
                if path not in manpage_paths[manpage]:
                    failed.setdefault(manpage, []).append(
                        "{} not found in manpage path {}".format(
                            path, manpage_paths[manpage]
                        )
                    )

            assert not failed, "One or more manpages failed:\n{}".format(
                pprint.pformat(failed)
            )



# -*- coding: utf-8 -*-
"""
Unit tests for the docker state
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.dockermod as docker_mod
import salt.states.docker_image as docker_state

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class DockerImageTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test docker_image states
    """

    def setup_loader_modules(self):
        return {
            docker_mod: {"__context__": {"docker.docker_version": ""}},
            docker_state: {"__opts__": {"test": False}},
        }

    def test_present_already_local(self):
        """
        According following sls,

        .. code-block:: yaml

            image:latest:
              docker_image.present:
                - force: true

        if ``image:latest`` is already downloaded locally the state
        should not report changes.
        """
        docker_inspect_image = MagicMock(return_value={"Id": "abcdefghijkl"})
        docker_pull = MagicMock(
            return_value={
                "Layers": {"Already_Pulled": ["abcdefghijkl"], "Pulled": []},
                "Status": "Image is up to date for image:latest",
                "Time_Elapsed": 1.1,
            }
        )
        docker_list_tags = MagicMock(return_value=["image:latest"])
        docker_resolve_tag = MagicMock(return_value="image:latest")
        __salt__ = {
            "docker.list_tags": docker_list_tags,
            "docker.pull": docker_pull,
            "docker.inspect_image": docker_inspect_image,
            "docker.resolve_tag": docker_resolve_tag,
        }
        with patch.dict(docker_state.__dict__, {"__salt__": __salt__}):
            ret = docker_state.present("image:latest", force=True)
            self.assertEqual(
                ret,
                {
                    "changes": {},
                    "result": True,
                    "comment": "Image 'image:latest' was pulled, "
                    "but there were no changes",
                    "name": "image:latest",
                },
            )

    def test_present_and_force(self):
        """
        According following sls,

        .. code-block:: yaml

            image:latest:
              docker_image.present:
                - force: true

        if ``image:latest`` is not downloaded and force is true
        should pull a new image successfully.
        """
        docker_inspect_image = MagicMock(return_value={"Id": "1234567890ab"})
        docker_pull = MagicMock(
            return_value={
                "Layers": {"Pulled": ["abcdefghijkl"]},
                "Status": "Image 'image:latest' was pulled",
                "Time_Elapsed": 1.1,
            }
        )
        docker_list_tags = MagicMock(side_effect=[[], ["image:latest"]])
        docker_resolve_tag = MagicMock(return_value="image:latest")
        __salt__ = {
            "docker.list_tags": docker_list_tags,
            "docker.pull": docker_pull,
            "docker.inspect_image": docker_inspect_image,
            "docker.resolve_tag": docker_resolve_tag,
        }
        with patch.dict(docker_state.__dict__, {"__salt__": __salt__}):
            ret = docker_state.present("image:latest", force=True)
        self.assertEqual(
            ret,
            {
                "changes": {
                    "Layers": {"Pulled": ["abcdefghijkl"]},
                    "Status": "Image 'image:latest' was pulled",
                    "Time_Elapsed": 1.1,
                },
                "result": True,
                "comment": "Image 'image:latest' was pulled",
                "name": "image:latest",
            },
        )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.sysctl as sysctl
from salt.exceptions import CommandExecutionError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class SysctlTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.sysctl
    """

    def setup_loader_modules(self):
        return {sysctl: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure that the named sysctl value is set
        in memory and persisted to the named configuration file.
        """
        name = "net.ipv4.conf.all.rp_filter"
        value = "1"
        config = "/etc/sysctl.conf"

        comment = (
            "Sysctl option {0} might be changed, we failed to check "
            "config file at {1}. The file is either unreadable, or "
            "missing.".format(name, config)
        )

        ret = {"name": name, "result": None, "changes": {}, "comment": comment}

        comment_empty = "Sysctl option {0} would be changed to {1}" "".format(
            name, value
        )

        comment1 = "Sysctl option {0} set to be changed to {1}".format(name, value)

        comment2 = (
            "Sysctl value is currently set on the running system but "
            "not in a config file. Sysctl option {0} set to be "
            "changed to 2 in config file.".format(name)
        )

        comt3 = (
            "Sysctl value {0} is present in configuration file but is not "
            "present in the running config. The value {0} is set to be "
            "changed to {1}".format(name, value)
        )

        comt4 = "Sysctl value {0} = {1} is already set".format(name, value)

        comt5 = "Sysctl option {0} would be changed to {1}".format(name, value)

        comt6 = "Failed to set {0} to {1}: ".format(name, value)

        comt7 = "Sysctl value {0} = {1} is already set".format(name, value)

        comt8 = "Sysctl value {0} = {1} was ignored".format(name, value)

        def mock_current(config_file=None):
            """
            Mock return value for __salt__.
            """
            if config_file is None:
                return {name: "2"}
            return [""]

        def mock_config(config_file=None):
            """
            Mock return value for __salt__.
            """
            if config_file is None:
                return {"salt": "2"}
            return [name]

        def mock_both(config_file=None):
            """
            Mock return value for __salt__.
            """
            if config_file is None:
                return {name: value}
            return [name]

        with patch.dict(sysctl.__opts__, {"test": True}):
            mock = MagicMock(return_value=None)
            with patch.dict(sysctl.__salt__, {"sysctl.show": mock}):
                self.assertDictEqual(sysctl.present(name, value), ret)

            mock = MagicMock(return_value=[])
            with patch.dict(sysctl.__salt__, {"sysctl.show": mock}):
                ret.update({"comment": comment_empty})
                self.assertDictEqual(sysctl.present(name, value), ret)

            with patch.dict(sysctl.__salt__, {"sysctl.show": mock_current}):
                ret.update({"comment": comment1})
                self.assertDictEqual(sysctl.present(name, value), ret)

                ret.update({"comment": comment2})
                self.assertDictEqual(sysctl.present(name, "2"), ret)

            with patch.dict(sysctl.__salt__, {"sysctl.show": mock_config}):
                ret.update({"comment": comt3})
                self.assertDictEqual(sysctl.present(name, value), ret)

            mock = MagicMock(return_value=value)
            with patch.dict(
                sysctl.__salt__, {"sysctl.show": mock_both, "sysctl.get": mock}
            ):
                ret.update({"comment": comt4, "result": True})
                self.assertDictEqual(sysctl.present(name, value), ret)

            mock = MagicMock(return_value=[True])
            with patch.dict(sysctl.__salt__, {"sysctl.show": mock}):
                ret.update({"comment": comt5, "result": None})
                self.assertDictEqual(sysctl.present(name, value), ret)

        with patch.dict(sysctl.__opts__, {"test": False}):
            mock = MagicMock(side_effect=CommandExecutionError)
            with patch.dict(sysctl.__salt__, {"sysctl.persist": mock}):
                ret.update({"comment": comt6, "result": False})
                self.assertDictEqual(sysctl.present(name, value), ret)

            mock = MagicMock(return_value="Already set")
            with patch.dict(sysctl.__salt__, {"sysctl.persist": mock}):
                ret.update({"comment": comt7, "result": True})
                self.assertDictEqual(sysctl.present(name, value), ret)

            mock = MagicMock(return_value="Ignored")
            with patch.dict(sysctl.__salt__, {"sysctl.persist": mock}):
                ret.update({"comment": comt8, "result": True})
                self.assertDictEqual(sysctl.present(name, value), ret)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Lukas Raska <lukas@raska.me>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.utils.dictdiffer as dictdiffer
from salt.exceptions import CommandExecutionError
from salt.states import elasticsearch

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class ElasticsearchTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.elasticsearch
    """

    def setup_loader_modules(self):
        return {
            elasticsearch: {
                "__opts__": {"test": False},
                "__utils__": {"dictdiffer.deep_diff": dictdiffer.deep_diff},
            }
        }

    # 'index_absent' function tests: 1

    def test_index_absent(self):
        """
        Test to manage a elasticsearch index.
        """
        name = "foo"

        ret = {
            "name": name,
            "result": True,
            "comment": "Index foo is already absent",
            "changes": {},
        }

        mock_get = MagicMock(
            side_effect=[
                None,
                {name: {"test": "key"}},
                {name: {}},
                {name: {"test": "key"}},
                CommandExecutionError,
                {name: {"test": "key"}},
            ]
        )
        mock_delete = MagicMock(side_effect=[True, False, CommandExecutionError])

        with patch.dict(
            elasticsearch.__salt__,
            {
                "elasticsearch.index_get": mock_get,
                "elasticsearch.index_delete": mock_delete,
            },
        ):
            self.assertDictEqual(elasticsearch.index_absent(name), ret)

            ret.update(
                {
                    "comment": "Successfully removed index foo",
                    "changes": {"old": {"test": "key"}},
                }
            )
            self.assertDictEqual(elasticsearch.index_absent(name), ret)

            ret.update(
                {
                    "comment": "Failed to remove index foo for unknown reasons",
                    "result": False,
                    "changes": {},
                }
            )
            self.assertDictEqual(elasticsearch.index_absent(name), ret)

            with patch.dict(elasticsearch.__opts__, {"test": True}):
                ret.update(
                    {
                        "comment": "Index foo will be removed",
                        "result": None,
                        "changes": {"old": {"test": "key"}},
                    }
                )
                self.assertDictEqual(elasticsearch.index_absent(name), ret)

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.index_absent(name), ret)

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.index_absent(name), ret)

    # 'index_present' function tests: 1

    def test_index_present(self):
        """
        Test to manage a elasticsearch index.
        """
        name = "foo"

        ret = {
            "name": name,
            "result": True,
            "comment": "Index foo is already present",
            "changes": {},
        }

        mock_exists = MagicMock(
            side_effect=[True, False, False, False, CommandExecutionError, False, False]
        )
        mock_get = MagicMock(
            side_effect=[{name: {"test": "key"}}, CommandExecutionError]
        )
        mock_create = MagicMock(side_effect=[True, False, CommandExecutionError, True])

        with patch.dict(
            elasticsearch.__salt__,
            {
                "elasticsearch.index_get": mock_get,
                "elasticsearch.index_exists": mock_exists,
                "elasticsearch.index_create": mock_create,
            },
        ):
            self.assertDictEqual(elasticsearch.index_present(name), ret)

            ret.update(
                {
                    "comment": "Successfully created index foo",
                    "changes": {"new": {"test": "key"}},
                }
            )
            self.assertDictEqual(elasticsearch.index_present(name), ret)

            ret.update(
                {
                    "comment": "Cannot create index foo, False",
                    "result": False,
                    "changes": {},
                }
            )
            self.assertDictEqual(elasticsearch.index_present(name), ret)

            with patch.dict(elasticsearch.__opts__, {"test": True}):
                ret.update(
                    {
                        "comment": "Index foo does not exist and will be created",
                        "result": None,
                        "changes": {"new": {"test2": "key"}},
                    }
                )
                self.assertDictEqual(
                    elasticsearch.index_present(name, {"test2": "key"}), ret
                )

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.index_absent(name), ret)

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.index_absent(name), ret)

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.index_absent(name), ret)

    # 'alias_absent' function tests: 1

    def test_alias_absent(self):
        """
        Test to manage a elasticsearch alias.
        """
        name = "foo"
        index = "bar"

        alias = {index: {"aliases": {name: {"test": "key"}}}}

        ret = {
            "name": name,
            "result": True,
            "comment": "Alias foo for index bar is already absent",
            "changes": {},
        }

        mock_get = MagicMock(
            side_effect=[
                None,
                {"foo2": {}},
                alias,
                alias,
                alias,
                CommandExecutionError,
                alias,
            ]
        )
        mock_delete = MagicMock(side_effect=[True, False, CommandExecutionError])

        with patch.dict(
            elasticsearch.__salt__,
            {
                "elasticsearch.alias_get": mock_get,
                "elasticsearch.alias_delete": mock_delete,
            },
        ):
            self.assertDictEqual(elasticsearch.alias_absent(name, index), ret)
            self.assertDictEqual(elasticsearch.alias_absent(name, index), ret)

            ret.update(
                {
                    "comment": "Successfully removed alias foo for index bar",
                    "changes": {"old": {"test": "key"}},
                }
            )
            self.assertDictEqual(elasticsearch.alias_absent(name, index), ret)

            ret.update(
                {
                    "comment": "Failed to remove alias foo for index bar for unknown reasons",
                    "result": False,
                    "changes": {},
                }
            )
            self.assertDictEqual(elasticsearch.alias_absent(name, index), ret)

            with patch.dict(elasticsearch.__opts__, {"test": True}):
                ret.update(
                    {
                        "comment": "Alias foo for index bar will be removed",
                        "result": None,
                        "changes": {"old": {"test": "key"}},
                    }
                )
                self.assertDictEqual(elasticsearch.alias_absent(name, index), ret)

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.alias_absent(name, index), ret)

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.alias_absent(name, index), ret)

    # 'alias_present' function tests: 1

    def test_alias_present(self):
        """
        Test to manage a elasticsearch alias.
        """
        name = "foo"
        index = "bar"

        alias = {index: {"aliases": {name: {"test": "key"}}}}

        ret = {
            "name": name,
            "result": True,
            "comment": "Alias foo for index bar is already present",
            "changes": {},
        }

        mock_get = MagicMock(
            side_effect=[
                alias,
                alias,
                None,
                None,
                None,
                alias,
                CommandExecutionError,
                None,
            ]
        )
        mock_create = MagicMock(side_effect=[True, True, False, CommandExecutionError])

        with patch.dict(
            elasticsearch.__salt__,
            {
                "elasticsearch.alias_get": mock_get,
                "elasticsearch.alias_create": mock_create,
            },
        ):
            self.assertDictEqual(
                elasticsearch.alias_present(name, index, {"test": "key"}), ret
            )

            ret.update(
                {
                    "comment": "Successfully replaced alias foo for index bar",
                    "changes": {"old": {"test": "key"}, "new": {"test2": "key"}},
                }
            )
            self.assertDictEqual(
                elasticsearch.alias_present(name, index, {"test2": "key"}), ret
            )

            ret.update(
                {
                    "comment": "Successfully created alias foo for index bar",
                    "changes": {"new": {"test2": "key"}},
                }
            )
            self.assertDictEqual(
                elasticsearch.alias_present(name, index, {"test2": "key"}), ret
            )

            ret.update(
                {
                    "comment": "Cannot create alias foo for index bar, False",
                    "result": False,
                }
            )
            self.assertDictEqual(
                elasticsearch.alias_present(name, index, {"test2": "key"}), ret
            )

            with patch.dict(elasticsearch.__opts__, {"test": True}):
                ret.update(
                    {
                        "comment": "Alias foo for index bar does not exist and will be created",
                        "result": None,
                        "changes": {"new": {"test2": "key"}},
                    }
                )
                self.assertDictEqual(
                    elasticsearch.alias_present(name, index, {"test2": "key"}), ret
                )

                ret.update(
                    {
                        "comment": "Alias foo for index bar exists with wrong configuration and will be overridden",
                        "result": None,
                        "changes": {"old": {"test": "key"}, "new": {"test2": "key"}},
                    }
                )
                self.assertDictEqual(
                    elasticsearch.alias_present(name, index, {"test2": "key"}), ret
                )

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.alias_present(name, index), ret)

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.alias_present(name, index), ret)

    # 'index_template_absent' function tests: 1

    def test_index_template_absent(self):
        """
        Test to manage a elasticsearch index template.
        """
        name = "foo"

        index_template = {name: {"test": "key"}}

        ret = {
            "name": name,
            "result": True,
            "comment": "Index template foo is already absent",
            "changes": {},
        }

        mock_get = MagicMock(
            side_effect=[
                None,
                {"bar": {}},
                index_template,
                index_template,
                index_template,
                CommandExecutionError,
                index_template,
            ]
        )
        mock_delete = MagicMock(side_effect=[True, False, CommandExecutionError])

        with patch.dict(
            elasticsearch.__salt__,
            {
                "elasticsearch.index_template_get": mock_get,
                "elasticsearch.index_template_delete": mock_delete,
            },
        ):
            self.assertDictEqual(elasticsearch.index_template_absent(name), ret)
            self.assertDictEqual(elasticsearch.index_template_absent(name), ret)

            ret.update(
                {
                    "comment": "Successfully removed index template foo",
                    "changes": {"old": {"test": "key"}},
                }
            )
            self.assertDictEqual(elasticsearch.index_template_absent(name), ret)

            ret.update(
                {
                    "comment": "Failed to remove index template foo for unknown reasons",
                    "result": False,
                    "changes": {},
                }
            )
            self.assertDictEqual(elasticsearch.index_template_absent(name), ret)

            with patch.dict(elasticsearch.__opts__, {"test": True}):
                ret.update(
                    {
                        "comment": "Index template foo will be removed",
                        "result": None,
                        "changes": {"old": {"test": "key"}},
                    }
                )
                self.assertDictEqual(elasticsearch.index_template_absent(name), ret)

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.index_template_absent(name), ret)

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.index_template_absent(name), ret)

    # 'index_template_present' function tests: 1

    def test_index_template_present(self):
        """
        Test to manage a elasticsearch index template.
        """
        name = "foo"

        index_template = {name: {"test": "key"}}

        ret = {
            "name": name,
            "result": True,
            "comment": "Index template foo is already present",
            "changes": {},
        }

        mock_exists = MagicMock(
            side_effect=[True, False, False, False, CommandExecutionError, False, False]
        )
        mock_create = MagicMock(side_effect=[True, False, CommandExecutionError, True])
        mock_get = MagicMock(side_effect=[index_template, CommandExecutionError])

        with patch.dict(
            elasticsearch.__salt__,
            {
                "elasticsearch.index_template_get": mock_get,
                "elasticsearch.index_template_create": mock_create,
                "elasticsearch.index_template_exists": mock_exists,
            },
        ):
            self.assertDictEqual(
                elasticsearch.index_template_present(name, {"test2": "key"}), ret
            )

            ret.update(
                {
                    "comment": "Successfully created index template foo",
                    "changes": {"new": {"test": "key"}},
                }
            )
            self.assertDictEqual(
                elasticsearch.index_template_present(name, {"test2": "key"}), ret
            )

            ret.update(
                {
                    "comment": "Cannot create index template foo, False",
                    "result": False,
                    "changes": {},
                }
            )
            self.assertDictEqual(
                elasticsearch.index_template_present(name, {"test2": "key"}), ret
            )

            with patch.dict(elasticsearch.__opts__, {"test": True}):
                ret.update(
                    {
                        "comment": "Index template foo does not exist and will be created",
                        "result": None,
                        "changes": {"new": {"test2": "key"}},
                    }
                )
                self.assertDictEqual(
                    elasticsearch.index_template_present(name, {"test2": "key"}), ret
                )

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.index_template_present(name, {}), ret)

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.index_template_present(name, {}), ret)

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.index_template_present(name, {}), ret)

    def test_index_template_present_check_definition(self):
        """
        Test to manage a elasticsearch index template.
        with check_definition set
        """
        name = "foo"

        index_template = {
            name: {"test2": "key", "aliases": {}, "mappings": {}, "settings": {}}
        }

        expected = {
            "name": name,
            "result": True,
            "comment": "Index template foo is already present and up to date",
            "changes": {},
        }

        mock_exists = MagicMock(side_effect=[True])
        mock_create = MagicMock(side_effect=[True])
        mock_get = MagicMock(side_effect=[index_template])

        with patch.dict(
            elasticsearch.__salt__,
            {
                "elasticsearch.index_template_get": mock_get,
                "elasticsearch.index_template_create": mock_create,
                "elasticsearch.index_template_exists": mock_exists,
            },
        ):

            ret = elasticsearch.index_template_present(
                name, {"test2": "key", "aliases": {}}, check_definition=True
            )
            self.assertDictEqual(expected, ret)

    def test_index_template_present_check_definition_alias_not_empty(self):
        """
        Test to manage a elasticsearch index template.
        with check_definition set and alias is not empty
        """
        name = "foo"

        index_template = {
            name: {"test2": "key", "aliases": {}, "mappings": {}, "settings": {}}
        }

        expected = {
            "name": name,
            "result": True,
            "comment": "Successfully updated index template foo",
            "changes": {"new": {"aliases": {"alias1": {}}}, "old": {"aliases": {}}},
        }

        mock_exists = MagicMock(side_effect=[True])
        mock_create = MagicMock(side_effect=[True])
        mock_get = MagicMock(side_effect=[index_template])

        with patch.dict(
            elasticsearch.__salt__,
            {
                "elasticsearch.index_template_get": mock_get,
                "elasticsearch.index_template_create": mock_create,
                "elasticsearch.index_template_exists": mock_exists,
            },
        ):

            ret = elasticsearch.index_template_present(
                name, {"test2": "key", "aliases": {"alias1": {}}}, check_definition=True
            )
            self.assertDictEqual(expected, ret)

    # 'pipeline_absent' function tests: 1

    def test_pipeline_absent(self):
        """
        Test to manage a elasticsearch pipeline.
        """
        name = "foo"

        pipeline = {name: {"test": "key"}}

        ret = {
            "name": name,
            "result": True,
            "comment": "Pipeline foo is already absent",
            "changes": {},
        }

        mock_get = MagicMock(
            side_effect=[
                None,
                {"foo2": {}},
                pipeline,
                pipeline,
                pipeline,
                CommandExecutionError,
                pipeline,
            ]
        )
        mock_delete = MagicMock(side_effect=[True, False, CommandExecutionError])

        with patch.dict(
            elasticsearch.__salt__,
            {
                "elasticsearch.pipeline_get": mock_get,
                "elasticsearch.pipeline_delete": mock_delete,
            },
        ):
            self.assertDictEqual(elasticsearch.pipeline_absent(name), ret)
            self.assertDictEqual(elasticsearch.pipeline_absent(name), ret)

            ret.update(
                {
                    "comment": "Successfully removed pipeline foo",
                    "changes": {"old": {"test": "key"}},
                }
            )
            self.assertDictEqual(elasticsearch.pipeline_absent(name), ret)

            ret.update(
                {
                    "comment": "Failed to remove pipeline foo for unknown reasons",
                    "result": False,
                    "changes": {},
                }
            )
            self.assertDictEqual(elasticsearch.pipeline_absent(name), ret)

            with patch.dict(elasticsearch.__opts__, {"test": True}):
                ret.update(
                    {
                        "comment": "Pipeline foo will be removed",
                        "result": None,
                        "changes": {"old": {"test": "key"}},
                    }
                )
                self.assertDictEqual(elasticsearch.pipeline_absent(name), ret)

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.pipeline_absent(name), ret)

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.pipeline_absent(name), ret)

    # 'pipeline_present' function tests: 1

    def test_pipeline_present(self):
        """
        Test to manage a elasticsearch pipeline.
        """
        name = "foo"

        pipeline = {name: {"test": "key"}}

        ret = {
            "name": name,
            "result": True,
            "comment": "Pipeline foo is already present",
            "changes": {},
        }

        mock_get = MagicMock(
            side_effect=[
                pipeline,
                pipeline,
                None,
                None,
                None,
                pipeline,
                CommandExecutionError,
                None,
            ]
        )
        mock_create = MagicMock(side_effect=[True, True, False, CommandExecutionError])

        with patch.dict(
            elasticsearch.__salt__,
            {
                "elasticsearch.pipeline_get": mock_get,
                "elasticsearch.pipeline_create": mock_create,
            },
        ):
            self.assertDictEqual(
                elasticsearch.pipeline_present(name, {"test": "key"}), ret
            )

            ret.update(
                {
                    "comment": "Successfully replaced pipeline foo",
                    "changes": {"old": {"test": "key"}, "new": {"test2": "key"}},
                }
            )
            self.assertDictEqual(
                elasticsearch.pipeline_present(name, {"test2": "key"}), ret
            )

            ret.update(
                {
                    "comment": "Successfully created pipeline foo",
                    "changes": {"new": {"test2": "key"}},
                }
            )
            self.assertDictEqual(
                elasticsearch.pipeline_present(name, {"test2": "key"}), ret
            )

            ret.update(
                {"comment": "Cannot create pipeline foo, False", "result": False}
            )
            self.assertDictEqual(
                elasticsearch.pipeline_present(name, {"test2": "key"}), ret
            )

            with patch.dict(elasticsearch.__opts__, {"test": True}):
                ret.update(
                    {
                        "comment": "Pipeline foo does not exist and will be created",
                        "result": None,
                        "changes": {"new": {"test2": "key"}},
                    }
                )
                self.assertDictEqual(
                    elasticsearch.pipeline_present(name, {"test2": "key"}), ret
                )

                ret.update(
                    {
                        "comment": "Pipeline foo exists with wrong configuration and will be overridden",
                        "result": None,
                        "changes": {"old": {"test": "key"}, "new": {"test2": "key"}},
                    }
                )
                self.assertDictEqual(
                    elasticsearch.pipeline_present(name, {"test2": "key"}), ret
                )

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.pipeline_present(name, {}), ret)

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.pipeline_present(name, {}), ret)

    # 'search_template_absent' function tests: 1

    def test_search_template_absent(self):
        """
        Test to manage a elasticsearch search template.
        """
        name = "foo"

        template = {"template": '{"test": "key"}'}

        ret = {
            "name": name,
            "result": True,
            "comment": "Search template foo is already absent",
            "changes": {},
        }

        mock_get = MagicMock(
            side_effect=[
                None,
                template,
                template,
                template,
                CommandExecutionError,
                template,
            ]
        )
        mock_delete = MagicMock(side_effect=[True, False, CommandExecutionError])

        with patch.dict(
            elasticsearch.__salt__,
            {
                "elasticsearch.search_template_get": mock_get,
                "elasticsearch.search_template_delete": mock_delete,
            },
        ):
            self.assertDictEqual(elasticsearch.search_template_absent(name), ret)

            ret.update(
                {
                    "comment": "Successfully removed search template foo",
                    "changes": {"old": {"test": "key"}},
                }
            )
            self.assertDictEqual(elasticsearch.search_template_absent(name), ret)

            ret.update(
                {
                    "comment": "Failed to remove search template foo for unknown reasons",
                    "result": False,
                    "changes": {},
                }
            )
            self.assertDictEqual(elasticsearch.search_template_absent(name), ret)

            with patch.dict(elasticsearch.__opts__, {"test": True}):
                ret.update(
                    {
                        "comment": "Search template foo will be removed",
                        "result": None,
                        "changes": {"old": {"test": "key"}},
                    }
                )
                self.assertDictEqual(elasticsearch.search_template_absent(name), ret)

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.search_template_absent(name), ret)

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.search_template_absent(name), ret)

    # 'pipeline_present' function tests: 1

    def test_search_template_present(self):
        """
        Test to manage a elasticsearch search template.
        """
        name = "foo"

        template = {"template": '{"test": "key"}'}

        ret = {
            "name": name,
            "result": True,
            "comment": "Search template foo is already present",
            "changes": {},
        }

        mock_get = MagicMock(
            side_effect=[
                template,
                template,
                None,
                None,
                None,
                template,
                CommandExecutionError,
                None,
            ]
        )
        mock_create = MagicMock(side_effect=[True, True, False, CommandExecutionError])

        with patch.dict(
            elasticsearch.__salt__,
            {
                "elasticsearch.search_template_get": mock_get,
                "elasticsearch.search_template_create": mock_create,
            },
        ):
            self.assertDictEqual(
                elasticsearch.search_template_present(name, {"test": "key"}), ret
            )

            ret.update(
                {
                    "comment": "Successfully replaced search template foo",
                    "changes": {"old": {"test": "key"}, "new": {"test2": "key"}},
                }
            )
            self.assertDictEqual(
                elasticsearch.search_template_present(name, {"test2": "key"}), ret
            )

            ret.update(
                {
                    "comment": "Successfully created search template foo",
                    "changes": {"new": {"test2": "key"}},
                }
            )
            self.assertDictEqual(
                elasticsearch.search_template_present(name, {"test2": "key"}), ret
            )

            ret.update(
                {"comment": "Cannot create search template foo, False", "result": False}
            )
            self.assertDictEqual(
                elasticsearch.search_template_present(name, {"test2": "key"}), ret
            )

            with patch.dict(elasticsearch.__opts__, {"test": True}):
                ret.update(
                    {
                        "comment": "Search template foo does not exist and will be created",
                        "result": None,
                        "changes": {"new": {"test2": "key"}},
                    }
                )
                self.assertDictEqual(
                    elasticsearch.search_template_present(name, {"test2": "key"}), ret
                )

                ret.update(
                    {
                        "comment": "Search template foo exists with wrong configuration and will be overridden",
                        "result": None,
                        "changes": {"old": {"test": "key"}, "new": {"test2": "key"}},
                    }
                )
                self.assertDictEqual(
                    elasticsearch.search_template_present(name, {"test2": "key"}), ret
                )

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.search_template_present(name, {}), ret)

            ret.update({"comment": "", "result": False, "changes": {}})
            self.assertDictEqual(elasticsearch.search_template_present(name, {}), ret)



# -*- coding: utf-8 -*-
#
# Author: Bo Maryniuk <bo@suse.de>
#
# Copyright 2017 SUSE LLC
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Import Salt Testing Libs
from __future__ import absolute_import, print_function, unicode_literals

import os
import sys

import salt.modules.ansiblegate as ansible
import salt.utils.platform
from salt.exceptions import LoaderError
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, MockTimedProc, patch
from tests.support.unit import TestCase, skipIf

try:
    import pytest
except ImportError as import_error:
    pytest = None
NO_PYTEST = not bool(pytest)


@skipIf(NO_PYTEST, False)
@skipIf(salt.utils.platform.is_windows(), "Not supported on Windows")
class AnsiblegateTestCase(TestCase, LoaderModuleMockMixin):
    def setUp(self):
        self.resolver = ansible.AnsibleModuleResolver({})
        self.resolver._modules_map = {
            "one.two.three": os.sep + os.path.join("one", "two", "three.py"),
            "four.five.six": os.sep + os.path.join("four", "five", "six.py"),
            "three.six.one": os.sep + os.path.join("three", "six", "one.py"),
        }

    def tearDown(self):
        self.resolver = None

    def setup_loader_modules(self):
        return {ansible: {}}

    def test_ansible_module_help(self):
        """
        Test help extraction from the module
        :return:
        """

        class Module(object):
            """
            An ansible module mock.
            """

            __name__ = "foo"
            DOCUMENTATION = """
---
one:
   text here
---
two:
   text here
description:
   describe the second part
        """

        with patch.object(ansible, "_resolver", self.resolver), patch.object(
            ansible._resolver, "load_module", MagicMock(return_value=Module())
        ):
            ret = ansible.help("dummy")
            assert sorted(
                ret.get('Available sections on module "{0}"'.format(Module().__name__))
            ) == ["one", "two"]
            assert ret.get("Description") == "describe the second part"

    def test_module_resolver_modlist(self):
        """
        Test Ansible resolver modules list.
        :return:
        """
        assert self.resolver.get_modules_list() == [
            "four.five.six",
            "one.two.three",
            "three.six.one",
        ]
        for ptr in ["five", "fi", "ve"]:
            assert self.resolver.get_modules_list(ptr) == ["four.five.six"]
        for ptr in ["si", "ix", "six"]:
            assert self.resolver.get_modules_list(ptr) == [
                "four.five.six",
                "three.six.one",
            ]
        assert self.resolver.get_modules_list("one") == [
            "one.two.three",
            "three.six.one",
        ]
        assert self.resolver.get_modules_list("one.two") == ["one.two.three"]
        assert self.resolver.get_modules_list("four") == ["four.five.six"]

    def test_resolver_module_loader_failure(self):
        """
        Test Ansible module loader.
        :return:
        """
        mod = "four.five.six"
        with pytest.raises(ImportError) as import_error:
            self.resolver.load_module(mod)

        mod = "i.even.do.not.exist.at.all"
        with pytest.raises(LoaderError) as loader_error:
            self.resolver.load_module(mod)

    def test_resolver_module_loader(self):
        """
        Test Ansible module loader.
        :return:
        """
        with patch("salt.modules.ansiblegate.importlib", MagicMock()), patch(
            "salt.modules.ansiblegate.importlib.import_module", lambda x: x
        ):
            assert (
                self.resolver.load_module("four.five.six")
                == "ansible.modules.four.five.six"
            )

    def test_resolver_module_loader_import_failure(self):
        """
        Test Ansible module loader failure.
        :return:
        """
        with patch("salt.modules.ansiblegate.importlib", MagicMock()), patch(
            "salt.modules.ansiblegate.importlib.import_module", lambda x: x
        ):
            with pytest.raises(LoaderError) as loader_error:
                self.resolver.load_module("something.strange")

    def test_virtual_function(self):
        """
        Test Ansible module __virtual__ when ansible is not installed on the minion.
        :return:
        """
        with patch("salt.modules.ansiblegate.ansible", None):
            assert ansible.__virtual__() == "ansible"

    def test_ansible_module_call(self):
        """
        Test Ansible module call from ansible gate module

        :return:
        """

        class Module(object):
            """
            An ansible module mock.
            """

            __name__ = "one.two.three"
            __file__ = "foofile"

            def main():  # pylint: disable=no-method-argument
                pass

        ANSIBLE_MODULE_ARGS = '{"ANSIBLE_MODULE_ARGS": ["arg_1", {"kwarg1": "foobar"}]}'

        proc = MagicMock(
            side_effect=[
                MockTimedProc(stdout=ANSIBLE_MODULE_ARGS.encode(), stderr=None),
                MockTimedProc(stdout='{"completed": true}'.encode(), stderr=None),
            ]
        )

        with patch.object(ansible, "_resolver", self.resolver), patch.object(
            ansible._resolver, "load_module", MagicMock(return_value=Module())
        ):
            _ansible_module_caller = ansible.AnsibleModuleCaller(ansible._resolver)
            with patch("salt.utils.timed_subprocess.TimedProc", proc):
                ret = _ansible_module_caller.call(
                    "one.two.three", "arg_1", kwarg1="foobar"
                )
                proc.assert_any_call(
                    [sys.executable, "foofile"],
                    stdin=ANSIBLE_MODULE_ARGS,
                    stdout=-1,
                    timeout=1200,
                )
                try:
                    proc.assert_any_call(
                        [
                            "echo",
                            '{"ANSIBLE_MODULE_ARGS": {"kwarg1": "foobar", "_raw_params": "arg_1"}}',
                        ],
                        stdout=-1,
                        timeout=1200,
                    )
                except AssertionError:
                    proc.assert_any_call(
                        [
                            "echo",
                            '{"ANSIBLE_MODULE_ARGS": {"_raw_params": "arg_1", "kwarg1": "foobar"}}',
                        ],
                        stdout=-1,
                        timeout=1200,
                    )
                assert ret == {"completed": True, "timeout": 1200}
